@@ -6,7 +6,6 @@ import time
 import math
 import binascii
 from copy import copy
-from operator import attrgetter
 import datetime
 
 from ..errors import Error
@@ -72,6 +71,9 @@ class Tag(object):
     DATE_TIME         = 0x21
 
 
+END_OF_CONTENTS_OCTETS = b'\x00\x00'
+
+
 class DecodeChoiceError(Error):
     pass
 
@@ -155,7 +157,6 @@ def encode_signed_integer(data):
     else:
         encoded.append(0)
 
-    encoded.append(len(encoded))
     encoded.reverse()
 
     return encoded
@@ -296,7 +297,7 @@ def decode_real_special(control):
 
 
 def decode_real_decimal(data):
-    return float(data[1:])
+    return float(data[1:].replace(b',', b'.'))
 
 
 def decode_real(data):
@@ -397,9 +398,18 @@ class Type(object):
 
         return end_offset
 
+    def set_default(self, value):
+        self.default = value
+
+    def get_default(self):
+        return self.default
+
     def is_default(self, value):
         return value == self.default
 
+    def has_default(self):
+        return self.default is not None
+
 
 class PrimitiveOrConstructedType(Type):
 
@@ -444,7 +454,7 @@ class PrimitiveOrConstructedType(Type):
             segments = []
 
             if length is None:
-                while data[offset:offset + 2] != b'\x00\x00':
+                while data[offset:offset + 2] != END_OF_CONTENTS_OCTETS:
                     decoded, offset = self.segment.decode(data, offset)
                     segments.append(decoded)
 
@@ -553,7 +563,7 @@ class MembersType(Type):
                 raise
         elif member.optional:
             pass
-        elif member.default is None:
+        elif not member.has_default():
             raise EncodeError("{} member '{}' not found in {}.".format(
                 self.__class__.__name__,
                 name,
@@ -563,15 +573,20 @@ class MembersType(Type):
         offset = self.decode_tag(data, offset)
 
         if data[offset] == 0x80:
-            raise NotImplementedError(
-                'Decode until an end-of-contents tag is not yet implemented.')
+            # Indefinite length field.
+            offset += 1
+            end_offset = None
         else:
+            # Definite length field
             length, offset = decode_length_definite(data, offset)
+            end_offset = offset + length
 
-        end_offset = offset + length
         values = {}
 
         for member in self.root_members:
+            # End of indefinite length sequence may be reached at any
+            # time, but DecodeError will occur (instead of usual
+            # IndexError) and so further members will be skipped.
             offset = self.decode_member(member,
                                         data,
                                         values,
@@ -579,10 +594,19 @@ class MembersType(Type):
                                         end_offset)
 
         if self.additions:
-            self.decode_additions(data,
-                                  values,
-                                  offset,
-                                  end_offset)
+            offset = self.decode_additions(data,
+                                           values,
+                                           offset,
+                                           end_offset)
+
+        # Detect end of indefinite length constructed field.
+        if end_offset is None:
+            if data[offset:offset + 2] == END_OF_CONTENTS_OCTETS:
+                end_offset = offset + 2
+            else:
+                raise DecodeError(
+                    'Could not find end-of-contents tag for indefinite length '
+                    'field.')
 
         return values, end_offset
 
@@ -609,27 +633,32 @@ class MembersType(Type):
         except DecodeError:
             pass
 
+        return offset
+
     def decode_member(self, member, data, values, offset, end_offset):
         try:
-            if offset < end_offset:
+            # If reached end of indefinite length field, member.decode
+            # will raise DecodeTagError, and end of field will be
+            # handled in MembersType.decode() method.
+            if end_offset is None or offset < end_offset:
                 if isinstance(member, AnyDefinedBy):
                     value, offset = member.decode(data, offset, values)
                 else:
                     value, offset = member.decode(data, offset)
             else:
                 raise IndexError
-        except (DecodeError, IndexError) as e:
+        except (DecodeError, DecodeTagError, IndexError) as e:
             if member.optional:
                 return offset
 
-            if member.default is None:
+            if not member.has_default():
                 if isinstance(e, IndexError):
                     e = DecodeError('out of data at offset {}'.format(offset))
 
                 e.location.append(member.name)
                 raise e
 
-            value = member.default
+            value = member.get_default()
 
         values[member.name] = value
 
@@ -667,17 +696,24 @@ class ArrayType(Type):
 
     def decode(self, data, offset):
         offset = self.decode_tag(data, offset)
-
         if data[offset] == 0x80:
-            raise NotImplementedError(
-                'Decode until an end-of-contents tag is not yet implemented.')
+            offset += 1
+            length = None  # Indicates indefinite field.
         else:
             length, offset = decode_length_definite(data, offset)
 
         decoded = []
         start_offset = offset
-
-        while (offset - start_offset) < length:
+        # Loop through data until length exceeded or end-of-contents tag reached.
+        while True:
+            if length is None:
+                # Find end of indefinite sequence.
+                if data[offset:offset + 2] == END_OF_CONTENTS_OCTETS:
+                    offset += 2
+                    break
+            elif (offset - start_offset) >= length:
+                # End of definite length sequence.
+                break
             decoded_element, offset = self.element_type.decode(data, offset)
             decoded.append(decoded_element)
 
@@ -726,7 +762,9 @@ class Integer(Type):
 
     def encode(self, data, encoded):
         encoded.extend(self.tag)
-        encoded.extend(encode_signed_integer(data))
+        value = encode_signed_integer(data)
+        encoded.extend(encode_length_definite(len(value)))
+        encoded.extend(value)
 
     def decode(self, data, offset):
         offset = self.decode_tag(data, offset)
@@ -922,7 +960,9 @@ class Enumerated(Type):
                     data))
 
         encoded.extend(self.tag)
-        encoded.extend(encode_signed_integer(value))
+        value = encode_signed_integer(value)
+        encoded.extend(encode_length_definite(len(value)))
+        encoded.extend(value)
 
     def decode(self, data, offset):
         offset = self.decode_tag(data, offset)
@@ -1345,6 +1385,18 @@ class ExplicitTag(Type):
         super(ExplicitTag, self).__init__(name, 'ExplicitTag', None)
         self.inner = inner
 
+    def set_default(self, value):
+        self.inner.set_default(value)
+
+    def get_default(self):
+        return self.inner.get_default()
+
+    def is_default(self, value):
+        return self.inner.is_default(value)
+
+    def has_default(self):
+        return self.inner.has_default()
+
     def set_tag(self, number, flags):
         super(ExplicitTag, self).set_tag(number,
                                          flags | Encoding.CONSTRUCTED)
@@ -1358,11 +1410,30 @@ class ExplicitTag(Type):
 
     def decode(self, data, offset):
         offset = self.decode_tag(data, offset)
-        _, offset = decode_length_definite(data, offset)
-        return self.inner.decode(data, offset)
+        indefinite = False
+
+        if data[offset] == 0x80:
+            # Indefinite length field
+            offset += 1
+            indefinite = True
+        else:
+            # Definite length field
+            length, offset = decode_length_definite(data, offset)
+
+        values, end_offset = self.inner.decode(data, offset)
+
+        if indefinite:
+            if data[end_offset:end_offset + 2] != END_OF_CONTENTS_OCTETS:
+                raise DecodeError(
+                    'Expected end-of-contents tag at offset: {}.'.format(
+                        end_offset))
+
+            end_offset += 2
+
+        return values, end_offset
 
     def __repr__(self):
-        return 'ExplicitTag()'
+        return 'ExplicitTag({})'.format(self.inner)
 
 
 class Recursive(Type, compiler.Recursive):
@@ -1401,14 +1472,6 @@ class Recursive(Type, compiler.Recursive):
 
 class CompiledType(compiler.CompiledType):
 
-    def __init__(self, type_):
-        super(CompiledType, self).__init__()
-        self._type = type_
-
-    @property
-    def type(self):
-        return self._type
-
     def encode(self, data):
         encoded = bytearray()
         self._type.encode(data, encoded)
@@ -1418,8 +1481,14 @@ class CompiledType(compiler.CompiledType):
     def decode(self, data):
         return self._type.decode(bytearray(data), 0)[0]
 
-    def __repr__(self):
-        return repr(self._type)
+    def decode_with_length(self, data):
+        return self._type.decode(bytearray(data), 0)
+
+
+def get_tag_no_encoding(member):
+    value = (member.tag[0] & ~Encoding.CONSTRUCTED)
+
+    return bytearray([value]) + member.tag[1:]
 
 
 class Compiler(compiler.Compiler):
@@ -1466,7 +1535,8 @@ class Compiler(compiler.Compiler):
             compiled = Real(name)
         elif type_name == 'ENUMERATED':
             compiled = Enumerated(name,
-                                  type_descriptor['values'],
+                                  self.get_enum_values(type_descriptor,
+                                                       module_name),
                                   self._numeric_enums)
         elif type_name == 'BOOLEAN':
             compiled = Boolean(name)
@@ -1544,6 +1614,7 @@ class Compiler(compiler.Compiler):
         return compiled
 
     def compile_type(self, name, type_descriptor, module_name):
+        module_name = self.get_module_name(type_descriptor, module_name)
         compiled = self.compile_implicit_type(name,
                                               type_descriptor,
                                               module_name)
@@ -1594,7 +1665,7 @@ class Compiler(compiler.Compiler):
                                          compiled_members)
 
         if sort_by_tag:
-            compiled_members = sorted(compiled_members, key=attrgetter('tag'))
+            compiled_members = sorted(compiled_members, key=get_tag_no_encoding)
 
         return compiled_members, additions
 
