@@ -83,9 +83,14 @@ def clean_bit_string_value(value, has_named_bits):
 
 class CompiledType(object):
 
-    def __init__(self):
+    def __init__(self, type_):
         self.constraints_checker = None
         self.type_checker = None
+        self._type = type_
+
+    @property
+    def type(self):
+        return self._type
 
     def check_types(self, data):
         return self.type_checker.encode(data)
@@ -93,6 +98,18 @@ class CompiledType(object):
     def check_constraints(self, data):
         return self.constraints_checker.encode(data)
 
+    def encode(self, data):
+        raise NotImplementedError('This codec does not support encode().')
+
+    def decode(self, data):
+        raise NotImplementedError('This codec does not support decode().')
+
+    def decode_with_length(self, data):
+        raise NotImplementedError('This codec does not support decode_with_length().')
+
+    def __repr__(self):
+        return repr(self._type)
+
 
 class Recursive(object):
     pass
@@ -140,26 +157,22 @@ class OpenTypeSequenceOf(object):
 class CompiledOpenTypes(CompiledType):
 
     def __init__(self, compiled_open_types, compiled_type):
-        super(CompiledOpenTypes, self).__init__()
+        super(CompiledOpenTypes, self).__init__(compiled_type)
         self._compiled_open_types = compiled_open_types
-        self._inner = compiled_type
 
     @property
     def type(self):
-        return self._inner.type
+        return self._type.type
 
     def encode(self, data, **kwargs):
         # print()
         # print('data:', data)
         # print(self._compiled_open_types)
 
-        return self._inner.encode(data, **kwargs)
+        return self._type.encode(data, **kwargs)
 
     def decode(self, data):
-        return self._inner.decode(data)
-
-    def __repr__(self):
-        return repr(self._inner)
+        return self._type.decode(data)
 
 
 class Compiler(object):
@@ -170,6 +183,7 @@ class Compiler(object):
         self._types_backtrace = []
         self.recursive_types = []
         self.compiled = {}
+        self.current_type_descriptor = None
 
     def types_backtrace_push(self, type_name):
         self._types_backtrace.append(type_name)
@@ -219,8 +233,7 @@ class Compiler(object):
         return compiled
 
     def pre_process(self):
-        for module_name in self._specification:
-            module = self._specification[module_name]
+        for module_name, module in self._specification.items():
             types = module['types']
             type_descriptors = types.values()
 
@@ -229,6 +242,15 @@ class Compiler(object):
             self.pre_process_tags(module, module_name)
             self.pre_process_default_value(type_descriptors, module_name)
 
+        for module_name, module in self._specification.items():
+            self.pre_process_parameterization_step_1(module['types'],
+                                                     module_name)
+
+        for module_name, module in self._specification.items():
+            types = module['types']
+            types = self.pre_process_parameterization_step_2(types)
+            module['types'] = types
+
         return self._specification
 
     def pre_process_components_of(self, type_descriptors, module_name):
@@ -308,10 +330,20 @@ class Compiler(object):
         module_tags = module.get('tags', 'EXPLICIT')
 
         for type_descriptor in module['types'].values():
+            self.current_type_descriptor = type_descriptor
             self.pre_process_tags_type(type_descriptor,
                                        module_tags,
                                        module_name)
 
+    def is_dummy_reference(self, type_name):
+        if 'parameters' not in self.current_type_descriptor:
+            return False
+
+        if type_name in self.current_type_descriptor['parameters']:
+            return True
+
+        return False
+
     def pre_process_tags_type(self,
                               type_descriptor,
                               module_tags,
@@ -325,6 +357,8 @@ class Compiler(object):
             if 'kind' not in tag:
                 if resolved_type_name == 'CHOICE':
                     tag['kind'] = 'EXPLICIT'
+                elif self.is_dummy_reference(type_name):
+                    tag['kind'] = 'EXPLICIT'
                 elif module_tags in ['IMPLICIT', 'EXPLICIT']:
                     tag['kind'] = module_tags
                 else:
@@ -433,9 +467,187 @@ class Compiler(object):
             mask >>= lowest_set_bit(mask)
             number_of_bits = len(default) - 2
 
-        mask = bitstruct.pack('u{}'.format(number_of_bits), mask)
+        if number_of_bits > 0:
+            mask = bitstruct.pack('u{}'.format(number_of_bits), mask)
+        else:
+            mask = b''
+
         member['default'] = (mask, number_of_bits)
 
+    def pre_process_parameterization_step_1(self, types, module_name):
+        """X.683 parameterization pre processing - step 1.
+
+        """
+
+        for type_name, type_descriptor in types.items():
+            # Skip if the type is parameterized.
+            if 'parameters' in type_descriptor:
+                continue
+
+            self.pre_process_parameterization_step_1_type(type_descriptor,
+                                                          type_name,
+                                                          module_name)
+
+    def pre_process_parameterization_step_1_type(self,
+                                                 type_descriptor,
+                                                 type_name,
+                                                 module_name):
+        """Recursively find and replace all parameterized types with their
+        actual types.
+
+        """
+
+        # SEQUENCE, SET and CHOICE.
+        if 'members' in type_descriptor:
+            for member in type_descriptor['members']:
+                if member == EXTENSION_MARKER:
+                    continue
+
+                self.pre_process_parameterization_step_1_type(member,
+                                                              type_name,
+                                                              module_name)
+
+        # SEQUENCE OF and SET OF.
+        if 'element' in type_descriptor:
+            self.pre_process_parameterization_step_1_type(
+                type_descriptor['element'],
+                type_name,
+                module_name)
+
+        # Just return if the type is not using a parameterized type.
+        if 'actual-parameters' not in type_descriptor:
+            return
+
+        (parameterized_type_descriptor,
+         parameterized_module_name) = self.lookup_type_descriptor(
+            type_descriptor['type'],
+            module_name)
+
+        if 'parameters' not in parameterized_type_descriptor:
+            raise CompileError(
+                "Type '{}' in module '{}' is not parameterized.".format(
+                    type_descriptor['type'],
+                    parameterized_module_name))
+
+        dummy_parameters = parameterized_type_descriptor['parameters']
+        actual_parameters = type_descriptor['actual-parameters']
+
+        if len(dummy_parameters) != len(actual_parameters):
+            raise CompileError(
+                "Parameterized type '{}' in module '{}' takes {} "
+                "parameters, but {} are given in type '{}' in "
+                "module '{}'.".format(type_descriptor['type'],
+                                      parameterized_module_name,
+                                      len(dummy_parameters),
+                                      len(actual_parameters),
+                                      type_name,
+                                      module_name))
+
+        parameterized_type_descriptor = deepcopy(parameterized_type_descriptor)
+
+        self.pre_process_parameterization_step_1_dummy_to_actual_type(
+            parameterized_type_descriptor,
+            dummy_parameters,
+            actual_parameters,
+            parameterized_module_name)
+
+        self.pre_process_parameterization_step_1_type(
+            parameterized_type_descriptor,
+            type_name,
+            parameterized_module_name)
+
+        type_descriptor.update(parameterized_type_descriptor)
+
+        if 'module-name' not in type_descriptor:
+            if module_name != parameterized_module_name:
+                type_descriptor['module-name'] = parameterized_module_name
+
+        if 'parameters' in type_descriptor:
+            del type_descriptor['parameters']
+
+        del type_descriptor['actual-parameters']
+
+    def pre_process_parameterization_step_1_dummy_to_actual_type(
+            self,
+            type_descriptor,
+            dummy_parameters,
+            actual_parameters,
+            module_name):
+        if 'members' in type_descriptor:
+            for member in type_descriptor['members']:
+                if member == EXTENSION_MARKER:
+                    continue
+
+                self.pre_process_parameterization_step_1_dummy_to_actual_type(
+                    member,
+                    dummy_parameters,
+                    actual_parameters,
+                    module_name)
+        elif 'element' in type_descriptor:
+            self.pre_process_parameterization_step_1_dummy_to_actual_type(
+                type_descriptor['element'],
+                dummy_parameters,
+                actual_parameters,
+                module_name)
+
+        # Replace dummy with actual in current type descriptor.
+        for dummy_parameter, actual_parameter in zip(dummy_parameters,
+                                                     actual_parameters):
+            if type_descriptor['type'] == dummy_parameter:
+                type_descriptor.update(actual_parameter)
+
+            if 'actual-parameters' in type_descriptor:
+                for i, parameter in enumerate(type_descriptor['actual-parameters']):
+                    if parameter['type'] == dummy_parameter:
+                        type_descriptor['actual-parameters'][i] = actual_parameter
+
+            if 'size' in type_descriptor:
+                actual_size = []
+
+                for item in type_descriptor['size']:
+                    if isinstance(item, tuple):
+                        minimum, maximum = item
+
+                        if minimum == dummy_parameter:
+                            minimum = actual_parameter
+
+                        if maximum == dummy_parameter:
+                            maximum = actual_parameter
+
+                        item = (minimum, maximum)
+                    elif item == dummy_parameter:
+                        item = actual_parameter
+
+                    actual_size.append(item)
+
+                type_descriptor['size'] = actual_size
+
+            if 'restricted-to' in type_descriptor:
+                actual_restricted_to = []
+
+                for minimum, maximum in type_descriptor['restricted-to']:
+                    if minimum == dummy_parameter:
+                        minimum = actual_parameter
+
+                    if maximum == dummy_parameter:
+                        maximum = actual_parameter
+
+                    actual_restricted_to.append((minimum, maximum))
+
+                type_descriptor['restricted-to'] = actual_restricted_to
+
+    def pre_process_parameterization_step_2(self, types):
+        """X.683 parameterization pre processing - step 2.
+
+        """
+
+        # Remove parameterized types as they are no longer needed.
+        return {
+            type_name: type_descriptor
+            for type_name, type_descriptor in types.items()
+            if 'parameters' not in type_descriptor
+        }
+
     def resolve_type_name(self, type_name, module_name):
         """Returns the ASN.1 type name of given type.
 
@@ -641,7 +853,7 @@ class Compiler(object):
 
         if 'default' in member:
             compiled_member = self.copy(compiled_member)
-            compiled_member.default = member['default']
+            compiled_member.set_default(member['default'])
 
         if 'size' in member:
             compiled_member = self.copy(compiled_member)
@@ -682,6 +894,22 @@ class Compiler(object):
 
         return minimum, maximum, has_extension_marker
 
+    def get_enum_values(self, type_descriptor, module_name):
+        """Converts the enum values to ints if they are value references.
+
+        """
+
+        enum_values = []
+
+        for value in type_descriptor['values']:
+            if value != EXTENSION_MARKER and not isinstance(value[1], int):
+                lookup = self.lookup_value(value[1], module_name)
+                enum_values.append((value[0], lookup[0]['value']))
+            else:
+                enum_values.append(value)
+
+        return enum_values
+
     def get_restricted_to_range(self, type_descriptor, module_name):
         def convert_value(value):
             try:
@@ -720,6 +948,20 @@ class Compiler(object):
     def get_with_components(self, type_descriptor):
         return type_descriptor.get('with-components', None)
 
+    def get_named_bits(self, type_descriptor, module_name):
+        named_bits = type_descriptor.get('named-bits', None)
+        if named_bits is not None:
+            named_bit_values = []
+
+            for value in named_bits:
+                if value != EXTENSION_MARKER and not value[1].isdigit():
+                    lookup = self.lookup_value(value[1], module_name)
+                    named_bit_values.append((value[0], lookup[0]['value']))
+                else:
+                    named_bit_values.append((value[0], int(value[1])))
+
+            return named_bit_values
+
     def is_explicit_tag(self, type_descriptor):
         try:
             return type_descriptor['tag']['kind'] == 'EXPLICIT'
@@ -728,6 +970,15 @@ class Compiler(object):
 
         return False
 
+    def get_module_name(self, type_descriptor, module_name):
+        module_name = type_descriptor.get('module-name', module_name)
+        try:
+            _, module_name = self.lookup_type_descriptor(type_descriptor['type'],
+                                                         module_name)
+        except CompileError:
+            pass
+        return module_name
+
     def lookup_in_modules(self, section, debug_string, name, module_name):
         begin_debug_string = debug_string[:1].upper() + debug_string[1:]
         module = self._specification[module_name]
