@@ -28,6 +28,12 @@ class Type(object):
     def set_size_range(self, minimum, maximum, has_extension_marker):
         pass
 
+    def set_default(self, value):
+        self.default = value
+
+    def has_default(self):
+        return self.default is not None
+
 
 class MembersType(Type):
 
@@ -57,7 +63,7 @@ class MembersType(Type):
                 encoded_members.append(encoded_member)
             elif member.optional:
                 pass
-            elif member.default is None:
+            elif not member.has_default():
                 raise EncodeError(
                     "{} member '{}' not found in {}.".format(
                         self.__class__.__name__,
@@ -493,14 +499,9 @@ class Recursive(Type, compiler.Recursive):
 class CompiledType(compiler.CompiledType):
 
     def __init__(self, type_name, compiled_type):
-        super(CompiledType, self).__init__()
+        super(CompiledType, self).__init__(compiled_type)
         self._value_name = type_name.lower()
         self._value_type = type_name
-        self._type = compiled_type
-
-    @property
-    def type(self):
-        return self._type
 
     def encode(self, data, indent=None):
         if indent is None:
@@ -517,9 +518,6 @@ class CompiledType(compiler.CompiledType):
     def decode(self, data):
         raise NotImplementedError('GSER decoding is not implemented.')
 
-    def __repr__(self):
-        return repr(self._type)
-
 
 class Compiler(compiler.Compiler):
 
@@ -531,6 +529,7 @@ class Compiler(compiler.Compiler):
         return CompiledType(type_name, compiled_type)
 
     def compile_type(self, name, type_descriptor, module_name):
+        module_name = self.get_module_name(type_descriptor, module_name)
         type_name = type_descriptor['type']
 
         if type_name == 'SEQUENCE':
@@ -564,7 +563,8 @@ class Compiler(compiler.Compiler):
             compiled = Real(name)
         elif type_name == 'ENUMERATED':
             compiled = Enumerated(name,
-                                  type_descriptor['values'],
+                                  self.get_enum_values(type_descriptor,
+                                                       module_name),
                                   self._numeric_enums)
         elif type_name == 'BOOLEAN':
             compiled = Boolean(name)
