@@ -31,6 +31,15 @@ class Type(object):
     def set_size_range(self, minimum, maximum, has_extension_marker):
         pass
 
+    def set_default(self, value):
+        self.default = value
+
+    def get_default(self):
+        return self.default
+
+    def has_default(self):
+        return self.default is not None
+
 
 class StringType(Type):
 
@@ -70,7 +79,7 @@ class MembersType(Type):
                 except EncodeError as e:
                     e.location.append(member.name)
                     raise
-            elif member.optional or member.default is not None:
+            elif member.optional or member.has_default():
                 continue
             else:
                 raise EncodeError(
@@ -94,8 +103,8 @@ class MembersType(Type):
                 values[name] = value
             elif member.optional:
                 pass
-            elif member.default is not None:
-                values[name] = member.default
+            elif member.has_default():
+                values[name] = member.get_default()
 
         return values
 
@@ -532,14 +541,6 @@ class Recursive(Type, compiler.Recursive):
 
 class CompiledType(compiler.CompiledType):
 
-    def __init__(self, type_):
-        super(CompiledType, self).__init__()
-        self._type = type_
-
-    @property
-    def type(self):
-        return self._type
-
     def encode(self, data, indent=None):
         dictionary = self._type.encode(data)
 
@@ -553,9 +554,6 @@ class CompiledType(compiler.CompiledType):
     def decode(self, data):
         return self._type.decode(json.loads(data.decode('utf-8')))
 
-    def __repr__(self):
-        return repr(self._type)
-
 
 class Compiler(compiler.Compiler):
 
@@ -567,6 +565,7 @@ class Compiler(compiler.Compiler):
         return CompiledType(compiled_type)
 
     def compile_type(self, name, type_descriptor, module_name):
+        module_name = self.get_module_name(type_descriptor, module_name)
         type_name = type_descriptor['type']
 
         if type_name == 'SEQUENCE':
@@ -600,7 +599,8 @@ class Compiler(compiler.Compiler):
             compiled = Real(name)
         elif type_name == 'ENUMERATED':
             compiled = Enumerated(name,
-                                  type_descriptor['values'],
+                                  self.get_enum_values(type_descriptor,
+                                                       module_name),
                                   self._numeric_enums)
         elif type_name == 'BOOLEAN':
             compiled = Boolean(name)
