@@ -161,6 +161,12 @@ class Encoder(object):
         self.append_length_determinant(number_of_bytes)
         self.append_non_negative_binary_integer(value, 8 * number_of_bytes)
 
+    def append_unsigned_integer(self, value):
+        number_of_bits = max(value.bit_length(), 1)
+        number_of_bytes = ((number_of_bits + 7) // 8)
+        self.append_length_determinant(number_of_bytes)
+        self.append_non_negative_binary_integer(value, 8 * number_of_bytes)
+
     def __repr__(self):
         return binascii.hexlify(self.as_bytearray()).decode('ascii')
 
@@ -262,6 +268,12 @@ class Decoder(object):
 
         return value
 
+    def read_unsigned_integer(self):
+        number_of_bytes = self.read_length_determinant()
+        number_of_bits = 8 * number_of_bytes
+
+        return self.read_non_negative_binary_integer(number_of_bits)
+
     def read_tag(self):
         byte = self.read_byte()
         tag = bytearray([byte])
@@ -304,9 +316,18 @@ class Type(object):
     def set_restricted_to_range(self, minimum, maximum, has_extension_marker):
         pass
 
+    def set_default(self, value):
+        self.default = value
+
+    def get_default(self):
+        return self.default
+
     def is_default(self, value):
         return value == self.default
 
+    def has_default(self):
+        return self.default is not None
+
 
 class KnownMultiplierStringType(Type):
 
@@ -404,7 +425,7 @@ class MembersType(Type):
                                    addition_encoder,
                                    encode_default=True)
 
-                if addition_encoder.number_of_bits > 0:
+                if addition_encoder.number_of_bits > 0 or addition.name in data:
                     addition_encoders.append(addition_encoder)
                     presence_bits |= 1
         except EncodeError:
@@ -417,7 +438,7 @@ class MembersType(Type):
         # Presence bit field.
         number_of_additions = len(self.additions)
         number_of_unused_bits = (8 - (number_of_additions % 8))
-        encoder.append_length_determinant(number_of_additions + 1)
+        encoder.append_length_determinant(((number_of_additions + 7) // 8) + 1)
         encoder.append_non_negative_binary_integer(number_of_unused_bits, 8)
         encoder.append_non_negative_binary_integer(presence_bits,
                                                    number_of_additions)
@@ -441,7 +462,7 @@ class MembersType(Type):
             except EncodeError as e:
                 e.location.append(member.name)
                 raise
-        elif member.optional or member.default is not None:
+        elif member.optional or member.has_default():
             pass
         else:
             raise EncodeError(
@@ -476,7 +497,7 @@ class MembersType(Type):
                 if optionals.get(member, True):
                     value = member.decode(decoder)
                     values[member.name] = value
-                elif member.default is not None:
+                elif member.has_default():
                     values[member.name] = member.default
             except DecodeError as e:
                 e.location.append(member.name)
@@ -487,13 +508,15 @@ class MembersType(Type):
     def decode_additions(self, decoder):
         # Presence bit field.
         length = decoder.read_length_determinant()
-        decoder.read_byte()
-        presence_bits = decoder.read_non_negative_binary_integer(length)
+        unused_bits = decoder.read_byte()
+        num_additions = ((length - 1) * 8) - unused_bits
+        presence_bits = decoder.read_non_negative_binary_integer(num_additions)
         decoder.align()
+
         decoded = {}
 
-        for i in range(length):
-            if presence_bits & (1 << (length - i - 1)):
+        for i in range(num_additions):
+            if presence_bits & (1 << (num_additions - i - 1)):
                 member_length = decoder.read_length_determinant()
 
                 if i < len(self.additions):
@@ -525,13 +548,13 @@ class ArrayType(Type):
         self.element_type = element_type
 
     def encode(self, data, encoder):
-        encoder.append_integer(len(data))
+        encoder.append_unsigned_integer(len(data))
 
         for entry in data:
             self.element_type.encode(entry, encoder)
 
     def decode(self, decoder):
-        length = decoder.read_integer()
+        length = decoder.read_unsigned_integer()
         decoded = []
 
         for _ in range(length):
@@ -572,10 +595,14 @@ class Integer(Type):
         self.has_extension_marker = False
         self.length = None
         self.fmt = None
+        self.signed = True
 
     def set_restricted_to_range(self, minimum, maximum, has_extension_marker):
         self.has_extension_marker = has_extension_marker
 
+        if minimum != 'MIN':
+            self.signed = (minimum < 0)
+
         if minimum == 'MIN' or maximum == 'MAX' or has_extension_marker:
             return
 
@@ -608,14 +635,18 @@ class Integer(Type):
     def encode(self, data, encoder):
         if self.fmt:
             encoder.append_bytes(struct.pack(self.fmt, data))
-        else:
+        elif self.signed:
             encoder.append_integer(data)
+        else:
+            encoder.append_unsigned_integer(data)
 
     def decode(self, decoder):
         if self.fmt:
             return struct.unpack(self.fmt, decoder.read_bytes(self.length))[0]
-        else:
+        elif self.signed:
             return decoder.read_integer()
+        else:
+            return decoder.read_unsigned_integer()
 
     def __repr__(self):
         return 'Integer({})'.format(self.name)
@@ -713,11 +744,12 @@ class Null(Type):
 
 class BitString(Type):
 
-    def __init__(self, name, minimum, maximum, has_extension_marker):
+    def __init__(self, name, named_bits, minimum, maximum, has_extension_marker):
         super(BitString, self).__init__(name,
                                         'BIT STRING',
                                         Tag.BIT_STRING)
         self.number_of_bits = None
+        self.named_bits = named_bits
 
         if minimum is not None or maximum is not None:
             if not has_extension_marker:
@@ -1257,14 +1289,6 @@ class Recursive(Type, compiler.Recursive):
 
 class CompiledType(compiler.CompiledType):
 
-    def __init__(self, type_):
-        super(CompiledType, self).__init__()
-        self._type = type_
-
-    @property
-    def type(self):
-        return self._type
-
     def encode(self, data):
         encoder = Encoder()
         self._type.encode(data, encoder)
@@ -1276,9 +1300,6 @@ class CompiledType(compiler.CompiledType):
 
         return self._type.decode(decoder)
 
-    def __repr__(self):
-        return repr(self._type)
-
 
 class Compiler(compiler.Compiler):
 
@@ -1290,6 +1311,7 @@ class Compiler(compiler.Compiler):
         return CompiledType(compiled_type)
 
     def compile_type(self, name, type_descriptor, module_name):
+        module_name = self.get_module_name(type_descriptor, module_name)
         type_name = type_descriptor['type']
 
         if type_name == 'SEQUENCE':
@@ -1322,7 +1344,8 @@ class Compiler(compiler.Compiler):
             compiled = Real(name, self.get_with_components(type_descriptor))
         elif type_name == 'ENUMERATED':
             compiled = Enumerated(name,
-                                  type_descriptor['values'],
+                                  self.get_enum_values(type_descriptor,
+                                                       module_name),
                                   self._numeric_enums)
         elif type_name == 'BOOLEAN':
             compiled = Boolean(name)
@@ -1374,6 +1397,8 @@ class Compiler(compiler.Compiler):
             compiled = DateTime(name)
         elif type_name == 'BIT STRING':
             compiled = BitString(name,
+                                 self.get_named_bits(type_descriptor,
+                                                     module_name),
                                  *self.get_size_range(type_descriptor,
                                                       module_name))
         elif type_name == 'ANY':
