@@ -19,6 +19,7 @@ from . import restricted_utc_time_from_datetime
 from . import restricted_generalized_time_to_datetime
 from . import restricted_generalized_time_from_datetime
 from .compiler import enum_values_split
+from .compiler import enum_values_as_dict
 from .compiler import clean_bit_string_value
 from .compiler import rstrip_bit_string_zeros
 from .ber import encode_real
@@ -28,11 +29,39 @@ from .ber import decode_object_identifier
 from .permitted_alphabet import NUMERIC_STRING
 from .permitted_alphabet import PRINTABLE_STRING
 from .permitted_alphabet import IA5_STRING
+from .permitted_alphabet import BMP_STRING
 from .permitted_alphabet import VISIBLE_STRING
 
 
 def is_unbound(minimum, maximum):
-    return minimum in [None, 'MIN'] or maximum in [None, 'MAX']
+    return ((minimum in [None, 'MIN'])
+            or (maximum in [None, 'MAX'])
+            or (maximum > 65535))
+
+
+def to_int(chars):
+    if isinstance(chars, int):
+        return chars
+
+    num = 0
+    byte_array = bytearray(chars)
+    while len(byte_array) > 0:
+        num <<= 8
+        byte = byte_array.pop(0)
+        num += byte
+
+    return num
+
+
+def to_byte_array(num, number_of_bits):
+    byte_array = bytearray()
+
+    while number_of_bits > 0:
+        byte_array.insert(0, (num & 0xff))
+        num >>= 8
+        number_of_bits -= 8
+
+    return byte_array
 
 
 def integer_as_number_of_bits(size):
@@ -525,12 +554,22 @@ class Type(object):
     def set_restricted_to_range(self, minimum, maximum, has_extension_marker):
         pass
 
+    def set_default(self, value):
+        self.default = value
+
+    def get_default(self):
+        return self.default
+
     def is_default(self, value):
         return value == self.default
 
+    def has_default(self):
+        return self.default is not None
+
 
 class KnownMultiplierStringType(Type):
 
+    ENCODING = 'ascii'
     PERMITTED_ALPHABET = PermittedAlphabet({}, {})
 
     def __init__(self,
@@ -565,40 +604,41 @@ class KnownMultiplierStringType(Type):
             self.number_of_bits = integer_as_number_of_bits(size)
 
     def encode(self, data, encoder):
-        encoded = bytearray(data.encode('ascii'))
 
         if self.has_extension_marker:
-            if self.minimum <= len(encoded) <= self.maximum:
+            if self.minimum <= len(data) <= self.maximum:
                 encoder.append_bit(0)
             else:
                 raise NotImplementedError(
                     'String size extension is not yet implemented.')
 
         if self.number_of_bits is None:
-            return self.encode_unbound(encoded, encoder)
+            return self.encode_unbound(data, encoder)
         elif self.minimum != self.maximum:
-            encoder.append_constrained_whole_number(len(encoded),
+            encoder.append_constrained_whole_number(len(data),
                                                     self.minimum,
                                                     self.maximum,
                                                     self.number_of_bits)
 
-            if self.maximum > 1 and len(encoded) > 0:
+            if self.maximum > 1 and len(data) > 0:
                 encoder.align()
         elif self.maximum * self.bits_per_character > 16:
             encoder.align()
 
-        for value in encoded:
+        for value in data:
             encoder.append_non_negative_binary_integer(
-                self.permitted_alphabet.encode(value),
+                self.permitted_alphabet.encode(
+                    to_int(value.encode(self.ENCODING))),
                 self.bits_per_character)
 
-    def encode_unbound(self, encoded, encoder):
+    def encode_unbound(self, data, encoder):
         encoder.align()
 
-        for offset, length in encoder.append_length_determinant_chunks(len(encoded)):
-            for entry in encoded[offset:offset + length]:
+        for offset, length in encoder.append_length_determinant_chunks(len(data)):
+            for entry in data[offset:offset + length]:
                 encoder.append_non_negative_binary_integer(
-                    self.permitted_alphabet.encode(entry),
+                    self.permitted_alphabet.encode(
+                        to_int(entry.encode(self.ENCODING))),
                     self.bits_per_character)
 
     def decode(self, decoder):
@@ -625,25 +665,29 @@ class KnownMultiplierStringType(Type):
             else:
                 length = self.minimum
 
-        data = []
+        data = bytearray()
 
         for _ in range(length):
             value = decoder.read_non_negative_binary_integer(self.bits_per_character)
-            data.append(self.permitted_alphabet.decode(value))
+            value = self.permitted_alphabet.decode(value)
+            data += to_byte_array(value, self.bits_per_character)
 
-        return bytearray(data).decode('ascii')
+        return data.decode(self.ENCODING)
 
     def decode_unbound(self, decoder):
         decoder.align()
-        decoded = []
+        decoded = bytearray()
+        orig_bits_per_character = integer_as_number_of_bits_power_of_two(
+            len(self.ALPHABET) - 1)
 
         for length in decoder.read_length_determinant_chunks():
             for _ in range(length):
                 value = decoder.read_non_negative_binary_integer(
                     self.bits_per_character)
-                decoded.append(self.permitted_alphabet.decode(value))
+                value = self.permitted_alphabet.decode(value)
+                decoded += to_byte_array(value, orig_bits_per_character)
 
-        return bytearray(decoded).decode('ascii')
+        return decoded.decode(self.ENCODING)
 
     def __repr__(self):
         return '{}({})'.format(self.__class__.__name__,
@@ -741,7 +785,7 @@ class MembersType(Type):
                                        addition_encoder,
                                        encode_default=True)
 
-                if addition_encoder.number_of_bits > 0:
+                if addition_encoder.number_of_bits > 0 or addition.name in data:
                     addition_encoders.append(addition_encoder)
                     presence_bits |= 1
         except EncodeError:
@@ -821,7 +865,7 @@ class MembersType(Type):
                 if optionals.get(member, True):
                     value = member.decode(decoder)
                     values[member.name] = value
-                elif member.default is not None:
+                elif member.has_default():
                     values[member.name] = member.default
             except DecodeError as e:
                 e.location.append(member.name)
@@ -1111,11 +1155,18 @@ class Null(Type):
 
 class BitString(Type):
 
-    def __init__(self, name, minimum, maximum, has_named_bits):
+    def __init__(self,
+                 name,
+                 named_bits,
+                 minimum,
+                 maximum,
+                 has_extension_marker):
         super(BitString, self).__init__(name, 'BIT STRING')
         self.minimum = minimum
         self.maximum = maximum
-        self.has_named_bits = has_named_bits
+        self.has_extension_marker = has_extension_marker
+        self.named_bits = named_bits
+        self.has_named_bits = named_bits is not None
 
         if is_unbound(minimum, maximum):
             self.number_of_bits = None
@@ -1145,6 +1196,13 @@ class BitString(Type):
     def encode(self, data, encoder):
         data, number_of_bits = data
 
+        if self.has_extension_marker:
+            if self.minimum <= number_of_bits <= self.maximum:
+                encoder.append_bit(0)
+            else:
+                raise NotImplementedError(
+                    'BIT STRING extension is not yet implemented.')
+
         if self.has_named_bits:
             data, number_of_bits = self.rstrip_zeros(data, number_of_bits)
 
@@ -1168,6 +1226,11 @@ class BitString(Type):
             encoder.append_bits(data[offset // 8:(offset + length + 7) // 8], length)
 
     def decode(self, decoder):
+        if self.has_extension_marker:
+            if decoder.read_bit():
+                raise NotImplementedError(
+                    'BIT STRING extension is not yet implemented.')
+
         if self.number_of_bits is None:
             return self.decode_unbound(decoder)
         else:
@@ -1338,6 +1401,11 @@ class Enumerated(Type):
         self.root_data_to_index = data_to_index
         self.root_number_of_bits = integer_as_number_of_bits(len(index_to_data) - 1)
 
+        if numeric:
+            self.root_data_to_value = {k: k for k in enum_values_as_dict(root)}
+        else:
+            self.root_data_to_value = {v: k for k, v in enum_values_as_dict(root).items()}
+
         # Optional additions.
         if additions is None:
             index_to_data = None
@@ -1485,7 +1553,14 @@ class Choice(Type):
         index_to_member, name_to_index = self.create_maps(root_members)
         self.root_index_to_member = index_to_member
         self.root_name_to_index = name_to_index
-        self.root_number_of_bits = integer_as_number_of_bits(len(root_members) - 1)
+        self.maximum = (len(root_members) - 1)
+        self.root_number_of_bits = integer_as_number_of_bits(self.maximum)
+
+        if self.maximum <= 65535:
+            self.number_of_indefinite_bits = None
+        else:
+            number_of_bits = ((self.root_number_of_bits + 7) // 8 - 1).bit_length()
+            self.number_of_indefinite_bits = number_of_bits
 
         # Optional additions.
         if additions is None:
@@ -1541,12 +1616,29 @@ class Choice(Type):
                     data[0]))
 
         if len(self.root_index_to_member) > 1:
-            encoder.append_non_negative_binary_integer(index,
-                                                       self.root_number_of_bits)
+            self.encode_root_index(index, encoder)
 
         member = self.root_index_to_member[index]
         self.encode_member(member, data[1], encoder)
 
+    def encode_root_index(self, index, encoder):
+        if self.number_of_indefinite_bits is None:
+            number_of_bits = self.root_number_of_bits
+        else:
+            number_of_bytes = size_as_number_of_bytes(index)
+            number_of_bits = 8 * number_of_bytes
+            encoder.append_constrained_whole_number(
+                number_of_bytes - 1,
+                0,
+                2 ** self.number_of_indefinite_bits,
+                self.number_of_indefinite_bits)
+            encoder.align()
+
+        encoder.append_constrained_whole_number(index,
+                                                0,
+                                                self.maximum,
+                                                number_of_bits)
+
     def encode_additions(self, data, encoder):
         try:
             index = self.additions_name_to_index[data[0]]
@@ -1586,8 +1678,7 @@ class Choice(Type):
 
     def decode_root(self, decoder):
         if len(self.root_index_to_member) > 1:
-            index = decoder.read_non_negative_binary_integer(
-                self.root_number_of_bits)
+            index = self.decode_root_index(decoder)
         else:
             index = 0
 
@@ -1601,6 +1692,22 @@ class Choice(Type):
 
         return (member.name, member.decode(decoder))
 
+    def decode_root_index(self, decoder):
+        if self.number_of_indefinite_bits is None:
+            number_of_bits = self.root_number_of_bits
+        else:
+            number_of_bytes = decoder.read_constrained_whole_number(
+                0,
+                2 ** self.number_of_indefinite_bits,
+                self.number_of_indefinite_bits)
+            number_of_bytes += 1
+            number_of_bits = (8 * number_of_bytes)
+            decoder.align()
+
+        return decoder.read_constrained_whole_number(0,
+                                                     self.maximum,
+                                                     number_of_bits)
+
     def decode_additions(self, decoder):
         index = decoder.read_normally_small_non_negative_whole_number()
 
@@ -1684,6 +1791,15 @@ class IA5String(KnownMultiplierStringType):
                                            ENCODE_DECODE_MAP)
 
 
+class BMPString(KnownMultiplierStringType):
+
+    ENCODING = 'utf-16-be'
+    ALPHABET = BMP_STRING
+    ENCODE_DECODE_MAP = {ord(v): ord(v) for v in ALPHABET}
+    PERMITTED_ALPHABET = PermittedAlphabet(ENCODE_DECODE_MAP,
+                                           ENCODE_DECODE_MAP)
+
+
 class VisibleString(KnownMultiplierStringType):
 
     ALPHABET = bytearray(VISIBLE_STRING.encode('ascii'))
@@ -1697,12 +1813,6 @@ class GeneralString(StringType):
     ENCODING = 'latin-1'
 
 
-class BMPString(StringType):
-
-    ENCODING = 'utf-16-be'
-    LENGTH_MULTIPLIER = 2
-
-
 class GraphicString(StringType):
 
     ENCODING = 'latin-1'
@@ -1918,14 +2028,6 @@ class AdditionGroup(Sequence):
 
 class CompiledType(compiler.CompiledType):
 
-    def __init__(self, type_):
-        super(CompiledType, self).__init__()
-        self._type = type_
-
-    @property
-    def type(self):
-        return self._type
-
     def encode(self, data):
         encoder = Encoder()
         self._type.encode(data, encoder)
@@ -1937,9 +2039,6 @@ class CompiledType(compiler.CompiledType):
 
         return self._type.decode(decoder)
 
-    def __repr__(self):
-        return repr(self._type)
-
 
 class Compiler(compiler.Compiler):
 
@@ -1951,6 +2050,7 @@ class Compiler(compiler.Compiler):
         return CompiledType(compiled_type)
 
     def compile_type(self, name, type_descriptor, module_name):
+        module_name = self.get_module_name(type_descriptor, module_name)
         type_name = type_descriptor['type']
 
         if type_name == 'SEQUENCE':
@@ -1990,7 +2090,8 @@ class Compiler(compiler.Compiler):
             compiled = Real(name)
         elif type_name == 'ENUMERATED':
             compiled = Enumerated(name,
-                                  type_descriptor['values'],
+                                  self.get_enum_values(type_descriptor,
+                                                       module_name),
                                   self._numeric_enums)
         elif type_name == 'BOOLEAN':
             compiled = Boolean(name)
@@ -2020,6 +2121,12 @@ class Compiler(compiler.Compiler):
                                  *self.get_size_range(type_descriptor,
                                                       module_name),
                                  permitted_alphabet=permitted_alphabet)
+        elif type_name == 'BMPString':
+            permitted_alphabet = self.get_permitted_alphabet(type_descriptor)
+            compiled = BMPString(name,
+                                 *self.get_size_range(type_descriptor,
+                                                      module_name),
+                                 permitted_alphabet=permitted_alphabet)
         elif type_name == 'VisibleString':
             permitted_alphabet = self.get_permitted_alphabet(type_descriptor)
             compiled = VisibleString(name,
@@ -2032,8 +2139,6 @@ class Compiler(compiler.Compiler):
             compiled = UTF8String(name)
         elif type_name == 'GraphicString':
             compiled = GraphicString(name)
-        elif type_name == 'BMPString':
-            compiled = BMPString(name)
         elif type_name == 'UTCTime':
             compiled = UTCTime(name)
         elif type_name == 'UniversalString':
@@ -2047,13 +2152,11 @@ class Compiler(compiler.Compiler):
         elif type_name == 'DATE-TIME':
             compiled = DateTime(name)
         elif type_name == 'BIT STRING':
-            minimum, maximum, _ = self.get_size_range(type_descriptor,
-                                                      module_name)
-            has_named_bits = ('named-bits' in type_descriptor)
             compiled = BitString(name,
-                                 minimum,
-                                 maximum,
-                                 has_named_bits)
+                                 self.get_named_bits(type_descriptor,
+                                                     module_name),
+                                 *self.get_size_range(type_descriptor,
+                                                      module_name))
         elif type_name == 'ANY':
             compiled = Any(name)
         elif type_name == 'ANY DEFINED BY':
