@@ -8,6 +8,8 @@ from . import restricted_utc_time_to_datetime
 from . import restricted_utc_time_from_datetime
 from . import restricted_generalized_time_to_datetime
 from . import restricted_generalized_time_from_datetime
+from .per import to_int
+from .per import to_byte_array
 from .per import integer_as_number_of_bits
 from .per import PermittedAlphabet
 from .per import Type
@@ -18,10 +20,8 @@ from .per import Enumerated
 from .per import ObjectIdentifier
 from .per import Sequence
 from .per import Set
-from .per import Choice
 from .per import UTF8String
 from .per import GeneralString
-from .per import BMPString
 from .per import GraphicString
 from .per import TeletexString
 from .per import UniversalString
@@ -31,6 +31,7 @@ from .per import Recursive
 from .permitted_alphabet import NUMERIC_STRING
 from .permitted_alphabet import PRINTABLE_STRING
 from .permitted_alphabet import IA5_STRING
+from .permitted_alphabet import BMP_STRING
 from .permitted_alphabet import VISIBLE_STRING
 
 
@@ -48,6 +49,7 @@ class Decoder(per.Decoder):
 
 class KnownMultiplierStringType(per.KnownMultiplierStringType):
 
+    ENCODING = 'ascii'
     PERMITTED_ALPHABET = ''
 
     def __init__(self,
@@ -68,20 +70,19 @@ class KnownMultiplierStringType(per.KnownMultiplierStringType):
             len(permitted_alphabet) - 1)
 
     def encode(self, data, encoder):
-        encoded = bytearray(data.encode('ascii'))
-
         if self.has_extension_marker:
             encoder.append_bit(0)
 
         if self.number_of_bits is None:
-            return self.encode_unbound(encoded, encoder)
+            return self.encode_unbound(data, encoder)
         elif self.minimum != self.maximum:
-            encoder.append_non_negative_binary_integer(len(encoded) - self.minimum,
+            encoder.append_non_negative_binary_integer(len(data) - self.minimum,
                                                        self.number_of_bits)
 
-        for value in encoded:
+        for value in data:
             encoder.append_non_negative_binary_integer(
-                self.permitted_alphabet.encode(value),
+                self.permitted_alphabet.encode(
+                    to_int(value.encode(self.ENCODING))),
                 self.bits_per_character)
 
     def decode(self, decoder):
@@ -100,13 +101,14 @@ class KnownMultiplierStringType(per.KnownMultiplierStringType):
             if self.minimum != self.maximum:
                 length += decoder.read_non_negative_binary_integer(self.number_of_bits)
 
-        data = []
+        data = bytearray()
 
         for _ in range(length):
             value = decoder.read_non_negative_binary_integer(self.bits_per_character)
-            data.append(self.permitted_alphabet.decode(value))
+            value = self.permitted_alphabet.decode(value)
+            data += to_byte_array(value, self.bits_per_character)
 
-        return bytearray(data).decode('ascii')
+        return data.decode(self.ENCODING)
 
 
 class ArrayType(per.ArrayType):
@@ -218,6 +220,13 @@ class BitString(per.BitString):
     def encode(self, data, encoder):
         data, number_of_bits = data
 
+        if self.has_extension_marker:
+            if self.minimum <= number_of_bits <= self.maximum:
+                encoder.append_bit(0)
+            else:
+                raise NotImplementedError(
+                    'BIT STRING extension is not yet implemented.')
+
         if self.has_named_bits:
             data, number_of_bits = self.rstrip_zeros(data, number_of_bits)
 
@@ -231,6 +240,11 @@ class BitString(per.BitString):
         encoder.append_bits(data, number_of_bits)
 
     def decode(self, decoder):
+        if self.has_extension_marker:
+            if decoder.read_bit():
+                raise NotImplementedError(
+                    'BIT STRING extension is not yet implemented.')
+
         if self.number_of_bits is None:
             return self.decode_unbound(decoder)
         else:
@@ -320,6 +334,15 @@ class SetOf(ArrayType):
                                     'SET OF')
 
 
+class Choice(per.Choice):
+
+    def encode_root_index(self, index, encoder):
+        encoder.append_non_negative_binary_integer(index, self.root_number_of_bits)
+
+    def decode_root_index(self, decoder):
+        return decoder.read_non_negative_binary_integer(self.root_number_of_bits)
+
+
 class NumericString(KnownMultiplierStringType):
 
     ALPHABET = bytearray(NUMERIC_STRING.encode('ascii'))
@@ -346,6 +369,15 @@ class IA5String(KnownMultiplierStringType):
                                            ENCODE_DECODE_MAP)
 
 
+class BMPString(KnownMultiplierStringType):
+
+    ENCODING = 'utf-16-be'
+    ALPHABET = BMP_STRING
+    ENCODE_DECODE_MAP = {ord(v): ord(v) for v in ALPHABET}
+    PERMITTED_ALPHABET = PermittedAlphabet(ENCODE_DECODE_MAP,
+                                           ENCODE_DECODE_MAP)
+
+
 class VisibleString(KnownMultiplierStringType):
 
     ALPHABET = bytearray(VISIBLE_STRING.encode('ascii'))
@@ -452,6 +484,7 @@ class Compiler(per.Compiler):
         return CompiledType(compiled_type)
 
     def compile_type(self, name, type_descriptor, module_name):
+        module_name = self.get_module_name(type_descriptor, module_name)
         type_name = type_descriptor['type']
 
         if type_name == 'SEQUENCE':
@@ -491,7 +524,8 @@ class Compiler(per.Compiler):
             compiled = Real(name)
         elif type_name == 'ENUMERATED':
             compiled = Enumerated(name,
-                                  type_descriptor['values'],
+                                  self.get_enum_values(type_descriptor,
+                                                       module_name),
                                   self._numeric_enums)
         elif type_name == 'BOOLEAN':
             compiled = Boolean(name)
@@ -521,6 +555,12 @@ class Compiler(per.Compiler):
                                  *self.get_size_range(type_descriptor,
                                                       module_name),
                                  permitted_alphabet=permitted_alphabet)
+        elif type_name == 'BMPString':
+            permitted_alphabet = self.get_permitted_alphabet(type_descriptor)
+            compiled = BMPString(name,
+                                 *self.get_size_range(type_descriptor,
+                                                      module_name),
+                                 permitted_alphabet=permitted_alphabet)
         elif type_name == 'VisibleString':
             permitted_alphabet = self.get_permitted_alphabet(type_descriptor)
             compiled = VisibleString(name,
@@ -533,8 +573,6 @@ class Compiler(per.Compiler):
             compiled = UTF8String(name)
         elif type_name == 'GraphicString':
             compiled = GraphicString(name)
-        elif type_name == 'BMPString':
-            compiled = BMPString(name)
         elif type_name == 'UTCTime':
             compiled = UTCTime(name)
         elif type_name == 'UniversalString':
@@ -548,13 +586,11 @@ class Compiler(per.Compiler):
         elif type_name == 'DATE-TIME':
             compiled = DateTime(name)
         elif type_name == 'BIT STRING':
-            minimum, maximum, _ = self.get_size_range(type_descriptor,
-                                                      module_name)
-            has_named_bits = ('named-bits' in type_descriptor)
             compiled = BitString(name,
-                                 minimum,
-                                 maximum,
-                                 has_named_bits)
+                                 self.get_named_bits(type_descriptor,
+                                                     module_name),
+                                 *self.get_size_range(type_descriptor,
+                                                      module_name))
         elif type_name == 'ANY':
             compiled = Any(name)
         elif type_name == 'ANY DEFINED BY':
