@@ -51,6 +51,15 @@ class Type(object):
     def set_size_range(self, minimum, maximum, has_extension_marker):
         pass
 
+    def set_default(self, value):
+        self.default = value
+
+    def get_default(self):
+        return self.default
+
+    def has_default(self):
+        return self.default is not None
+
     def encode(self, data):
         raise NotImplementedError('To be implemented by subclasses.')
 
@@ -112,7 +121,7 @@ class MembersType(Type):
                 except EncodeError as e:
                     e.location.append(member.name)
                     raise
-            elif member.optional or member.default is not None:
+            elif member.optional or member.has_default():
                 continue
             else:
                 raise EncodeError(
@@ -137,8 +146,8 @@ class MembersType(Type):
                 values[name] = value
             elif member.optional:
                 pass
-            elif member.default is not None:
-                values[name] = member.default
+            elif member.has_default():
+                values[name] = member.get_default()
 
         return values
 
@@ -670,14 +679,6 @@ class Recursive(Type, compiler.Recursive):
 
 class CompiledType(compiler.CompiledType):
 
-    def __init__(self, type_):
-        super(CompiledType, self).__init__()
-        self._type = type_
-
-    @property
-    def type(self):
-        return self._type
-
     def encode(self, data, indent=None):
         element = self._type.encode(data)
 
@@ -691,9 +692,6 @@ class CompiledType(compiler.CompiledType):
 
         return self._type.decode(element)
 
-    def __repr__(self):
-        return repr(self._type)
-
 
 class Compiler(compiler.Compiler):
 
@@ -705,6 +703,7 @@ class Compiler(compiler.Compiler):
         return CompiledType(compiled_type)
 
     def compile_type(self, name, type_descriptor, module_name):
+        module_name = self.get_module_name(type_descriptor, module_name)
         type_name = type_descriptor['type']
 
         if type_name == 'SEQUENCE':
@@ -740,7 +739,8 @@ class Compiler(compiler.Compiler):
             compiled = Real(name)
         elif type_name == 'ENUMERATED':
             compiled = Enumerated(name,
-                                  type_descriptor['values'],
+                                  self.get_enum_values(type_descriptor,
+                                                       module_name),
                                   self._numeric_enums)
         elif type_name == 'BOOLEAN':
             compiled = Boolean(name)
