@@ -166,7 +166,7 @@ def convert_enum_values(string, location, tokens):
             if not isinstance(item, list):
                 continue
 
-            item_number = int(item[2])
+            item_number = item[2]
 
             if item_number in used_numbers:
                 raise ParseError(
@@ -181,7 +181,7 @@ def convert_enum_values(string, location, tokens):
 
     for token in root:
         if isinstance(token, list):
-            values.append((token[0], int(token[2])))
+            values.append((token[0], token[2]))
         else:
             while number in used_numbers:
                 number += 1
@@ -198,7 +198,7 @@ def convert_enum_values(string, location, tokens):
 
         for token in additional:
             if isinstance(token, list):
-                number = int(token[2])
+                number = token[2]
                 values.append((token[0], number))
             else:
                 if number in used_numbers:
@@ -337,7 +337,7 @@ def convert_members(tokens):
         else:
             qualifiers = []
 
-        member = convert_type(member_tokens[2])
+        member = convert_type(member_tokens[2], [])
         member['name'] = member_tokens[0]
 
         if 'OPTIONAL' in qualifiers:
@@ -371,7 +371,7 @@ def convert_sequence_type(_s, _l, tokens):
 def convert_sequence_of_type(_s, _l, tokens):
     converted_type = {
         'type': 'SEQUENCE OF',
-        'element': convert_type(tokens[4]),
+        'element': convert_type(tokens[4], []),
     }
 
     if len(tokens[1]) > 0:
@@ -398,7 +398,7 @@ def convert_set_type(_s, _l, tokens):
 def convert_set_of_type(_s, _l, tokens):
     converted_type = {
         'type': 'SET OF',
-        'element': convert_type(tokens[4])
+        'element': convert_type(tokens[4], [])
     }
 
     if len(tokens[1]) > 0:
@@ -423,10 +423,16 @@ def convert_choice_type(_s, _l, tokens):
 
 
 def convert_defined_type(_s, _l, tokens):
-    return {
+    converted = {
         'type': tokens[0]
     }
 
+    # actual-parameters
+    if len(tokens) == 2:
+        converted.update(tokens[1])
+
+    return converted
+
 
 def convert_integer_type(_s, _l, tokens):
     tokens = tokens.asList()
@@ -502,6 +508,32 @@ def convert_any_defined_by_type(_s, _l, tokens):
     }
 
 
+def convert_actual_parameter_list(_s, _l, tokens):
+    tokens = tokens.asList()[0]
+    converted = []
+
+    for parameter in tokens:
+        converted.append(parameter[0][0])
+
+    if converted:
+        converted = {'actual-parameters': converted}
+    else:
+        converted = None
+
+    return converted
+
+
+def convert_parameter_list(_s, _l, tokens):
+    tokens = tokens.asList()
+
+    if tokens[0]:
+        converted = {'parameters': tokens[0]}
+    else:
+        converted = None
+
+    return converted
+
+
 def convert_null_type(_s, _l, _tokens):
     return {
         'type': 'NULL'
@@ -514,7 +546,7 @@ def convert_boolean_type(_s, _l, _tokens):
     }
 
 
-def convert_type(tokens):
+def convert_type(tokens, parameters):
     converted_type, constraints = tokens
 
     restricted_to = []
@@ -539,6 +571,9 @@ def convert_type(tokens):
             elif 'with-components' in constraint_tokens[0]:
                 converted_type.update(constraint_tokens[0])
 
+    if isinstance(parameters, dict):
+        converted_type.update(parameters)
+
     if '{' in restricted_to:
         restricted_to = []
 
@@ -549,6 +584,7 @@ def convert_type(tokens):
         'BIT STRING',
         'OCTET STRING',
         'IA5String',
+        'BMPString',
         'VisibleString',
         'UTF8String',
         'NumericString',
@@ -686,10 +722,10 @@ def convert_parameterized_object_class_assignment(_s, _l, tokens):
 
 def convert_parameterized_type_assignment(_s, _l, tokens):
     tokens = tokens.asList()
-    converted_type = convert_type(tokens[3])
+    converted_type = convert_type(tokens[4], tokens[1])
 
     try:
-        tag = convert_tag(tokens[2])
+        tag = convert_tag(tokens[3])
     except ValueError:
         tag = None
 
@@ -726,8 +762,15 @@ def convert_imports(_s, _l, tokens):
     if tokens:
         for from_tokens in tokens:
             from_name = from_tokens[2]
+
             LOGGER.debug("Converting imports from '%s'.", from_name)
-            imports[from_name] = from_tokens[0]
+
+            imported = from_tokens[0]
+
+            if from_name in imports:
+                imported = list(set(imports[from_name] + imported))
+
+            imports[from_name] = sorted(imported)
 
     return {'imports': imports}
 
@@ -977,10 +1020,10 @@ def create_grammar():
     dummy_governor = dummy_reference
     governor = (type_ | defined_object_class)
     param_governor = (governor | dummy_governor)
-    parameter = (Optional(param_governor + colon) + dummy_reference)
-    parameter_list = Suppress(Optional(left_brace
-                                       + delimitedList(parameter)
-                                       + right_brace))
+    parameter = (Suppress(Optional(param_governor + colon)) + dummy_reference)
+    parameter_list = Group(Optional(Suppress(left_brace)
+                                    + delimitedList(parameter)
+                                    + Suppress(right_brace)))
 
     # X.683: 9. Referencing parameterized definitions
     actual_parameter = Group(type_
@@ -1110,7 +1153,7 @@ def create_grammar():
                     # | defined_object_class
                     | parameterized_object_class)
     parameterized_object_class_assignment = (object_class_reference
-                                             + parameter_list
+                                             + Suppress(parameter_list)
                                              + assign
                                              + object_class)
 
@@ -1139,7 +1182,7 @@ def create_grammar():
                  | object_from_object
                  | parameterized_object)
     parameterized_object_assignment = (object_reference
-                                       + parameter_list
+                                       + Suppress(parameter_list)
                                        + defined_object_class
                                        + Suppress(assign)
                                        + object_)
@@ -1158,7 +1201,7 @@ def create_grammar():
     object_set <<= (left_brace + Group(object_set_spec) + right_brace)
     object_set.setName('"{"')
     parameterized_object_set_assignment = (object_set_reference
-                                           + parameter_list
+                                           + Suppress(parameter_list)
                                            + defined_object_class
                                            - assign
                                            - object_set)
@@ -1584,7 +1627,7 @@ def create_grammar():
                                      - tag
                                      - type_)
     parameterized_value_assignment = (value_reference
-                                      + parameter_list
+                                      + Suppress(parameter_list)
                                       - Group(type_)
                                       - Suppress(assign)
                                       - value)
@@ -1708,6 +1751,8 @@ def create_grammar():
     character_string_type.setParseAction(convert_keyword_type)
     object_class_field_type.setParseAction(convert_keyword_type)
     any_defined_by_type.setParseAction(convert_any_defined_by_type)
+    actual_parameter_list.setParseAction(convert_actual_parameter_list)
+    parameter_list.setParseAction(convert_parameter_list)
 
     return specification
 
