@@ -1,7 +1,7 @@
 EXPECTED = {'PKIXTSP': {'extensibility-implied': False,
              'imports': {'CryptographicMessageSyntax': ['ContentInfo'],
-                         'PKIX1Explicit88': ['Extensions',
-                                             'AlgorithmIdentifier'],
+                         'PKIX1Explicit88': ['AlgorithmIdentifier',
+                                             'Extensions'],
                          'PKIX1Implicit88': ['GeneralName'],
                          'PKIXCMP': ['PKIFreeText']},
              'object-classes': {},
