@@ -1,11 +1,17 @@
+import sys
 import json
 import unittest
 from datetime import date
 from datetime import time
 from datetime import datetime
+from copy import deepcopy
 from .utils import Asn1ToolsBaseTest
 import asn1tools
 
+sys.path.append('tests/files')
+
+from parameterization import EXPECTED as PARAMETERIZATION
+
 
 CODECS = ['ber', 'der', 'jer', 'oer', 'per', 'uper', 'xer']
 ALL_CODECS = CODECS + ['gser']
@@ -689,7 +695,7 @@ class Asn1ToolsCodecsConsistencyTest(Asn1ToolsBaseTest):
 
         for codec in CODECS:
             specs.append(asn1tools.compile_files([
-                'tests/files/c_source.asn',
+                'tests/files/c_source/c_source.asn',
                 'examples/programming_types/programming_types.asn'
             ], codec))
 
@@ -841,30 +847,75 @@ class Asn1ToolsCodecsConsistencyTest(Asn1ToolsBaseTest):
                     'p': {
                         'q': 5 * b'\x03',
                         'r': True
-                    }
+                    },
+                    's': True
                 }
             }
         ]
 
         encoded_messages = [
             b'\x30\x30\x30\x2e\xa0\x0f\xa0\x03\x80\x01\x00\xa1\x06\x05\x00\x05'
-            b'\x00\x05\x00\x82\x00\xa1\x07\x80\x01\x01\x81\x02\x54\x55\xa2\x12'
-            b'\x80\x01\x00\x81\x01\x02\xa2\x0a\x80\x05\x03\x03\x03\x03\x03\x81'
+            b'\x00\x05\x00\x82\x00\xa1\x04\x81\x02\x54\x55\xa2\x15\x80\x01\x00'
+            b'\x81\x01\x02\xa2\x0a\x80\x05\x03\x03\x03\x03\x03\x81\x01\xff\x83'
             b'\x01\xff',
             b'\x30\x30\x30\x2e\xa0\x0f\xa0\x03\x80\x01\x00\xa1\x06\x05\x00\x05'
-            b'\x00\x05\x00\x82\x00\xa1\x07\x80\x01\x01\x81\x02\x54\x55\xa2\x12'
-            b'\x80\x01\x00\x81\x01\x02\xa2\x0a\x80\x05\x03\x03\x03\x03\x03\x81'
+            b'\x00\x05\x00\x82\x00\xa1\x04\x81\x02\x54\x55\xa2\x15\x80\x01\x00'
+            b'\x81\x01\x02\xa2\x0a\x80\x05\x03\x03\x03\x03\x03\x81\x01\xff\x83'
             b'\x01\xff',
-            b'[{"m":{"p":{"q":"0303030303","r":true},"o":2,"n":false},"g":{"l"'
-            b':"5455","h":"j"},"a":{"b":{"c":0},"e":[null,null,null],"f":null}'
-            b'}]',
-            b'\x01\x01\x80\x00\x01\x03\x01\x02\x54\x55\xe0\x00\x02\x80\x03\x03'
-            b'\x03\x03\x03\xff',
-            b'\x00\xc0\x54\x55\xe9\x03\x03\x03\x03\x03\x80',
-            b'\x00\xd5\x15\x7a\x40\xc0\xc0\xc0\xc0\xe0',
+            b'[{"m":{"p":{"q":"0303030303","r":true},"s":true,"o":2,"n":false}'
+            b',"g":{"l":"5455","h":"j"},"a":{"b":{"c":0},"e":[null,null,null],'
+            b'"f":null}}]',
+            b'\x01\x01\x80\x00\x01\x03\x00\x02\x54\x55\xf0\x00\x02\x80\x03\x03'
+            b'\x03\x03\x03\xff\xff',
+            b'\x00\x80\x54\x55\xf4\x80\x03\x03\x03\x03\x03\xc0',
+            b'\x00\xaa\x2a\xfa\x40\xc0\xc0\xc0\xc0\xf0',
             b'<D><SEQUENCE><a><b><c>0</c></b><e><NULL /><NULL /><NULL /></e><f'
             b' /></a><g><h><j /></h><l>5455</l></g><m><n><false /></n><o>2</o>'
-            b'<p><q>0303030303</q><r><true /></r></p></m></SEQUENCE></D>'
+            b'<p><q>0303030303</q><r><true /></r></p><s><true /></s></m></SEQU'
+            b'ENCE></D>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'D', decoded, encoded)
+
+        # Type D - some missing.
+        decoded = [
+            {
+                'a': {
+                    'b': ('d', False),
+                    'e': [None, None, None],
+                    'f': None
+                },
+                'g': {
+                    'h': 'k',
+                    'l': b'\x54'
+                },
+                'm': {
+                    'o': 3,
+                    'p': {
+                        'q': 5 * b'\x03'
+                    },
+                    's': False
+                }
+            }
+        ]
+
+        encoded_messages = [
+            b'\x30\x27\x30\x25\xa0\x0f\xa0\x03\x81\x01\x00\xa1\x06\x05\x00\x05'
+            b'\x00\x05\x00\x82\x00\xa1\x07\x80\x02\x02\x00\x81\x01\x54\xa2\x09'
+            b'\xa2\x07\x80\x05\x03\x03\x03\x03\x03',
+            b'\x30\x27\x30\x25\xa0\x0f\xa0\x03\x81\x01\x00\xa1\x06\x05\x00\x05'
+            b'\x00\x05\x00\x82\x00\xa1\x07\x80\x02\x02\x00\x81\x01\x54\xa2\x09'
+            b'\xa2\x07\x80\x05\x03\x03\x03\x03\x03',
+            b'[{"a":{"b":{"d":false},"e":[null,null,null],"f":null},"g":{"h":"'
+            b'k","l":"54"},"m":{"o":3,"p":{"q":"0303030303"},"s":false}}]',
+            b'\x01\x01\x81\x00\x01\x03\x80\x82\x02\x00\x01\x54\x20\x00\x03\x03'
+            b'\x03\x03\x03',
+            b'\x09\x80\x54\x20\x03\x03\x03\x03\x03',
+            b'\x09\x8a\x84\x03\x03\x03\x03\x03',
+            b'<D><SEQUENCE><a><b><d><false /></d></b><e><NULL /><NULL /><NULL '
+            b'/></e><f /></a><g><h><k /></h><l>54</l></g><m><o>3</o><p><q>0303'
+            b'030303</q></p><s><false /></s></m></SEQUENCE></D>'
         ]
 
         for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
@@ -1031,6 +1082,658 @@ class Asn1ToolsCodecsConsistencyTest(Asn1ToolsBaseTest):
         for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
             self.encode_decode_codec(spec, codec, 'O', decoded, encoded)
 
+        # Type Q.
+        decoded = ('c256', True)
+
+        encoded_messages = [
+            b'\x9f\x81\x7f\x01\xff',
+            b'\x9f\x81\x7f\x01\xff',
+            b'{"c256": true}',
+            b'\xbf\x81\x7f\xff',
+            b'\x00\xff\x80',
+            b'\x7f\xc0',
+            b'<Q><c256><true /></c256></Q>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'Q', decoded, encoded)
+
+        # Type Q.
+        decoded = ('c257', True)
+
+        encoded_messages = [
+            b'\x9f\x82\x00\x01\xff',
+            b'\x9f\x82\x00\x01\xff',
+            b'{"c257": true}',
+            b'\xbf\x82\x00\xff',
+            b'\x01\x00\x80',
+            b'\x80\x40',
+            b'<Q><c257><true /></c257></Q>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'Q', decoded, encoded)
+
+        # Type R.
+        decoded = -1
+
+        encoded_messages = [
+            b'\x02\x01\xff',
+            b'\x02\x01\xff',
+            b'-1',
+            b'\xff',
+            b'\x00',
+            b'\x00',
+            b'<R>-1</R>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'R', decoded, encoded)
+
+        decoded = 0
+
+        encoded_messages = [
+            b'\x02\x01\x00',
+            b'\x02\x01\x00',
+            b'0',
+            b'\x00',
+            b'\x80',
+            b'\x80',
+            b'<R>0</R>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'R', decoded, encoded)
+
+        # Type S.
+        decoded = -2
+
+        encoded_messages = [
+            b'\x02\x01\xfe',
+            b'\x02\x01\xfe',
+            b'-2',
+            b'\xfe',
+            b'\x00',
+            b'\x00',
+            b'<S>-2</S>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'S', decoded, encoded)
+
+        decoded = 1
+
+        encoded_messages = [
+            b'\x02\x01\x01',
+            b'\x02\x01\x01',
+            b'1',
+            b'\x01',
+            b'\xc0',
+            b'\xc0',
+            b'<S>1</S>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'S', decoded, encoded)
+
+        # Type T.
+        decoded = -1
+
+        encoded_messages = [
+            b'\x02\x01\xff',
+            b'\x02\x01\xff',
+            b'-1',
+            b'\xff',
+            b'\x00',
+            b'\x00',
+            b'<T>-1</T>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'T', decoded, encoded)
+
+        decoded = 2
+
+        encoded_messages = [
+            b'\x02\x01\x02',
+            b'\x02\x01\x02',
+            b'2',
+            b'\x02',
+            b'\xc0',
+            b'\xc0',
+            b'<T>2</T>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'T', decoded, encoded)
+
+        # Type U.
+        decoded = -64
+
+        encoded_messages = [
+            b'\x02\x01\xc0',
+            b'\x02\x01\xc0',
+            b'-64',
+            b'\xc0',
+            b'\x00',
+            b'\x00',
+            b'<U>-64</U>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'U', decoded, encoded)
+
+        # Type V.
+        decoded = -128
+
+        encoded_messages = [
+            b'\x02\x01\x80',
+            b'\x02\x01\x80',
+            b'-128',
+            b'\x80',
+            b'\x00',
+            b'\x00',
+            b'<V>-128</V>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'V', decoded, encoded)
+
+        # Type W.
+        decoded = -1
+
+        encoded_messages = [
+            b'\x02\x01\xff',
+            b'\x02\x01\xff',
+            b'-1',
+            b'\xff\xff',
+            b'\x00\x00',
+            b'\x00\x00',
+            b'<W>-1</W>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'W', decoded, encoded)
+
+        decoded = 510
+
+        encoded_messages = [
+            b'\x02\x02\x01\xfe',
+            b'\x02\x02\x01\xfe',
+            b'510',
+            b'\x01\xfe',
+            b'\x01\xff',
+            b'\xff\x80',
+            b'<W>510</W>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'W', decoded, encoded)
+
+        # Type X.
+        decoded = -2
+
+        encoded_messages = [
+            b'\x02\x01\xfe',
+            b'\x02\x01\xfe',
+            b'-2',
+            b'\xff\xfe',
+            b'\x00\x00',
+            b'\x00\x00',
+            b'<X>-2</X>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'X', decoded, encoded)
+
+        decoded = 510
+
+        encoded_messages = [
+            b'\x02\x02\x01\xfe',
+            b'\x02\x02\x01\xfe',
+            b'510',
+            b'\x01\xfe',
+            b'\x02\x00',
+            b'\x80\x00',
+            b'<X>510</X>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'X', decoded, encoded)
+
+        # Type Y.
+        decoded = 10000
+
+        encoded_messages = [
+            b'\x02\x02\x27\x10',
+            b'\x02\x02\x27\x10',
+            b'10000',
+            b'\x27\x10',
+            b'\x00\x00',
+            b'\x00\x00',
+            b'<Y>10000</Y>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'Y', decoded, encoded)
+
+        decoded = 10512
+
+        encoded_messages = [
+            b'\x02\x02\x29\x10',
+            b'\x02\x02\x29\x10',
+            b'10512',
+            b'\x29\x10',
+            b'\x02\x00',
+            b'\x80\x00',
+            b'<Y>10512</Y>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'Y', decoded, encoded)
+
+        # AB.
+        decoded = {
+            'a': 0,
+            'b': 10300
+        }
+
+        encoded_messages = [
+            b'\x30\x0b\xa0\x03\x02\x01\x00\xa1\x04\x02\x02\x28\x3c',
+            b'\x30\x0b\xa0\x03\x02\x01\x00\xa1\x04\x02\x02\x28\x3c',
+            b'{"a": 0, "b": 10300}',
+            b'\x00\x28\x3c',
+            b'\x80\x01,',
+            b'\xa5\x80',
+            b'<AB><a>0</a><b>10300</b></AB>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'AB', decoded, encoded)
+
+        # AE
+        decoded = {
+            'a': False,
+            'b': True,
+            'c': False
+        }
+
+        encoded_messages = [
+            b'\x30\x06\x80\x01\x00\x82\x01\x00',
+            b'\x30\x06\x80\x01\x00\x82\x01\x00',
+            b'{"a": false, "b": true, "c": false}',
+            b'\x40\x00\x00',
+            b'\x40',
+            b'\x40',
+            b'<AE><a><false /></a><b><true /></b><c><false /></c></AE>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'AE', decoded, encoded)
+
+        # AF
+        decoded = {
+            'a': True,
+            'b': {
+                'c': True,
+                'd': 17,
+                'e': 'g'
+            },
+            'e': 18,
+            'f': 19,
+            'g': 20,
+            'h': 21,
+            'i': 22,
+            'j': 23,
+            'k': 24,
+            'l': 25,
+        }
+
+        encoded_messages = [
+            b'\x30\x23\x80\x01\xff\xa1\x09\x80\x01\xff\x81\x01\x11\x82\x01\x01'
+            b'\x82\x01\x12\x83\x01\x13\x84\x01\x14\x85\x01\x15\x86\x01\x16\x87'
+            b'\x01\x17\x89\x01\x19',
+            b'\x30\x23\x80\x01\xff\xa1\x09\x80\x01\xff\x81\x01\x11\x82\x01\x01'
+            b'\x82\x01\x12\x83\x01\x13\x84\x01\x14\x85\x01\x15\x86\x01\x16\x87'
+            b'\x01\x17\x89\x01\x19',
+            b'{"a": true, "b": {"c":true, "d":17, "e":"g"}, "e":18, "f":19, "g":20,'
+            b'"h":21, "i":22, "j":23, "k":24, "l":25}',
+            b'\x80\xff\x03\x07\xff\x80\x09\x80\xff\x02\x06\xc0\x01\x11\x01\x01'
+            b'\x01\x12\x01\x13\x01\x14\x01\x15\x01\x16\x01\x17\x01\x18\x01\x19',
+            b'\xc4\x7f\xc0\x06\xc0\xe0\x01\x11\x01\x40\x01\x12\x01\x13\x01\x14'
+            b'\x01\x15\x01\x16\x01\x17\x01\x18\x01\x19',
+            b'\xc4\x7f\xc1\xb0\x38\x08\x88\x0a\x00\x00\x44\x80\x44\xc0\x45\x00'
+            b'\x45\x40\x45\x80\x45\xc0\x46\x00\x46\x40',
+            b'<AF><a><true /></a><b><c><true /></c><d>17</d><e><g /></e></b><e>18'
+            b'</e><f>19</f><g>20</g><h>21</h><i>22</i><j>23</j><k>24</k><l>25'
+            b'</l></AF>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'AF', decoded, encoded)
+
+        # Type AG
+        decoded = {
+            'a': True,
+            'b': b'\x84\x55',
+            'c': [True, False, True, False],
+            'd': 'f',
+            'h': None,
+            'i': 1.0,
+            'j': ('k', 60693),
+            'm': b'\xf0\xf1\xf2\xf3\xf4'
+        }
+
+        encoded_messages = [
+            b'\x30\x2e\x80\x01\xff\x81\x02\x84\x55\xa2\x0c\x01\x01\xff\x01\x01'
+            b'\x00\x01\x01\xff\x01\x01\x00\x83\x02\x01\x00\x84\x00\x85\x03\x80'
+            b'\x00\x01\xa6\x05\x80\x03\x00\xed\x15\x87\x05\xf0\xf1\xf2\xf3\xf4',
+            b'\x30\x2e\x80\x01\xff\x81\x02\x84\x55\xa2\x0c\x01\x01\xff\x01\x01'
+            b'\x00\x01\x01\xff\x01\x01\x00\x83\x02\x01\x00\x84\x00\x85\x03\x80'
+            b'\x00\x01\xa6\x05\x80\x03\x00\xed\x15\x87\x05\xf0\xf1\xf2\xf3\xf4',
+            b'{"a": true, "b": "8455", "c": [true, false, true, false], "d": "f",'
+            b'"h": null, "i": 1.0, "j": {"k": 60693}, "m": "F0F1F2F3F4"}',
+            b'\x80\xff\x02\x01\xfe\x03\x02\x84\x55\x06\x01\x04\xff\x00\xff\x00'
+            b'\x03\x82\x01\x00\x00\x04\x3f\x80\x00\x00\x03\x80\xed\x15\x05\xf0'
+            b'\xf1\xf2\xf3\xf4',
+            b'\xc3\x7f\x03\x20\x84\x55\x01\x3a\x01\x40\x00\x04\x03\x80\x00\x01'
+            b'\x03\x00\xed\x15\x05\xf0\xf1\xf2\xf3\xf4',
+            b'\xc3\x7f\x03\x28\x45\x50\x01\x3a\x01\x40\x00\x04\x03\x80\x00\x01'
+            b'\x03\x76\x8a\x80\x05\xf0\xf1\xf2\xf3\xf4',
+            b'<AG><a><true /></a><b>8455</b><c><true /><false /><true /><false />'
+            b'</c><d><f /></d><h /><i>1.0E0</i><j><k>60693</k></j><m>F0F1F2F3F4</'
+            b'm></AG>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'AG', decoded, encoded)
+
+        # Type AL.
+        decoded = -129
+
+        encoded_messages = [
+            b'\x02\x02\xff\x7f',
+            b'\x02\x02\xff\x7f',
+            b'-129',
+            b'\xff\x7f',
+            b'\x00\x00',
+            b'\x00\x00',
+            b'<AL>-129</AL>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'AL', decoded, encoded)
+
+        decoded = 127
+
+        encoded_messages = [
+            b'\x02\x01\x7f',
+            b'\x02\x01\x7f',
+            b'127',
+            b'\x00\x7f',
+            b'\x01\x00',
+            b'\x80\x00',
+            b'<AL>127</AL>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'AL', decoded, encoded)
+
+        # Type AM.
+        decoded = -2
+
+        encoded_messages = [
+            b'\x02\x01\xfe',
+            b'\x02\x01\xfe',
+            b'-2',
+            b'\xff\xfe',
+            b'\x00',
+            b'\x00',
+            b'<AM>-2</AM>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'AM', decoded, encoded)
+
+        decoded = 128
+
+        encoded_messages = [
+            b'\x02\x02\x00\x80',
+            b'\x02\x02\x00\x80',
+            b'128',
+            b'\x00\x80',
+            b'\x82',
+            b'\x82',
+            b'<AM>128</AM>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'AM', decoded, encoded)
+
+        # Type AQ.
+        decoded = 1234
+
+        encoded_messages = [
+            b'\x02\x02\x04\xd2',
+            b'\x02\x02\x04\xd2',
+            b'1234',
+            b'\x00\x00\x04\xd2',
+            b'\x40\x04\xd2',
+            b'\x00\x04\xd2',
+            b'<AQ>1234</AQ>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'AQ', decoded, encoded)
+
+        decoded = 100001
+
+        encoded_messages = [
+            b'\x02\x03\x01\x86\xa1',
+            b'\x02\x03\x01\x86\xa1',
+            b'100001',
+            b'\x00\x01\x86\xa1',
+            b'\x80\x01\x86\xa1',
+            b'\x01\x86\xa1',
+            b'<AQ>100001</AQ>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'AQ', decoded, encoded)
+
+    def test_parameterization(self):
+        specs = []
+
+        for codec in CODECS:
+            specs.append(asn1tools.compile_dict(deepcopy(PARAMETERIZATION),
+                                                codec))
+
+        # A-Boolean.
+        decoded = {'a': True}
+
+        encoded_messages = [
+            b'\x30\x05\xa0\x03\x01\x01\xff',
+            b'\x30\x05\xa0\x03\x01\x01\xff',
+            b'{"a": true}',
+            b'\xff',
+            b'\x80',
+            b'\x80',
+            b'<A-Boolean><a><true /></a></A-Boolean>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'A-Boolean', decoded, encoded)
+
+        # A-Integer.
+        decoded = {'a': 555}
+
+        encoded_messages = [
+            b'\x30\x06\xa0\x04\x02\x02\x02\x2b',
+            b'\x30\x06\xa0\x04\x02\x02\x02\x2b',
+            b'{"a": 555}',
+            b'\x02\x02\x2b',
+            b'\x02\x02\x2b',
+            b'\x02\x02\x2b',
+            b'<A-Integer><a>555</a></A-Integer>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'A-Integer', decoded, encoded)
+
+        # B-BooleanInteger.
+        decoded = {
+            'a': True,
+            'b': -40000
+        }
+
+        encoded_messages = [
+            b'\x30\x0c\xa0\x03\x01\x01\xff\xa1\x05\x02\x03\xff\x63\xc0',
+            b'\x30\x0c\xa0\x03\x01\x01\xff\xa1\x05\x02\x03\xff\x63\xc0',
+            b'{"a": true, "b": -40000}',
+            b'\x80\xff\x03\xff\x63\xc0',
+            b'\xc0\x03\xff\x63\xc0',
+            b'\xc0\xff\xd8\xf0\x00',
+            b'<B-BooleanInteger><a><true /></a><b>-40000</b></B-BooleanInteger>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec,
+                                     codec,
+                                     'B-BooleanInteger',
+                                     decoded,
+                                     encoded)
+
+        # D.
+        decoded = {
+            'a': ('a', {'a': True}),
+            'b': ('c', {'a': {'a': None, 'b': 5}})
+        }
+
+        encoded_messages = [
+            b'\x30\x1c\xa0\x09\xa0\x07\x30\x05\xa0\x03\x01\x01\xff\xa1\x0f\xa0'
+            b'\x0d\xa0\x0b\x30\x09\xa0\x02\x05\x00\xa1\x03\x02\x01\x05',
+            b'\x30\x1c\xa0\x09\xa0\x07\x30\x05\xa0\x03\x01\x01\xff\xa1\x0f\xa0'
+            b'\x0d\xa0\x0b\x30\x09\xa0\x02\x05\x00\xa1\x03\x02\x01\x05',
+            b'{"a": {"a": {"a": true}}, "b": {"c": {"a": {"a": null, "b": 5}}}'
+            b'}',
+            b'\x80\x00\xff\x80\x80\x01\x05',
+            b'\x28\x01\x05',
+            b'\x28\x08\x28',
+            b'<D><a><a><a><true /></a></a></a><b><c><a><a /><b>5</b></a></c></'
+            b'b></D>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'D', decoded, encoded)
+
+        decoded = {
+            'a': ('a', {'a': True}),
+            'b': ('d', {'a': None, 'b': 5})
+        }
+
+        encoded_messages = [
+            b'\x30\x18\xa0\x09\xa0\x07\x30\x05\xa0\x03\x01\x01\xff\xa1\x0b\xa1'
+            b'\x09\xa0\x02\x05\x00\xa1\x03\x02\x01\x05',
+            b'\x30\x18\xa0\x09\xa0\x07\x30\x05\xa0\x03\x01\x01\xff\xa1\x0b\xa1'
+            b'\x09\xa0\x02\x05\x00\xa1\x03\x02\x01\x05',
+            b'{"a": {"a": {"a": true}}, "b": {"d": {"a": null, "b": 5}}}',
+            b'\x80\x00\xff\x81\x80\x01\x05',
+            b'\x38\x01\x05',
+            b'\x38\x08\x28',
+            b'<D><a><a><a><true /></a></a></a><b><d><a /><b>5</b></d></b></D>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'D', decoded, encoded)
+
+        decoded = {
+            'a': ('b', {'a': {'a': True}}),
+            'b': ('d', {'a': None, 'b': 5})
+        }
+
+        encoded_messages = [
+            b'\x30\x1a\xa0\x0b\xa1\x09\xa0\x07\x30\x05\xa0\x03\x01\x01\xff\xa1'
+            b'\x0b\xa1\x09\xa0\x02\x05\x00\xa1\x03\x02\x01\x05',
+            b'\x30\x1a\xa0\x0b\xa1\x09\xa0\x07\x30\x05\xa0\x03\x01\x01\xff\xa1'
+            b'\x0b\xa1\x09\xa0\x02\x05\x00\xa1\x03\x02\x01\x05',
+            b'{"a": {"b": {"a": {"a": true}}}, "b": {"d": {"a": null, "b": 5}}}',
+            b'\x81\x00\x00\xff\x81\x80\x01\x05',
+            b'\x9c\x01\x05',
+            b'\x9c\x04\x14',
+            b'<D><a><b><a><a><true /></a></a></b></a><b><d><a /><b>5</b></d></b'
+            b'></D>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'D', decoded, encoded)
+
+        # F.
+        decoded = [('b', {'c': [{'a': 2, 'b': True}]})]
+
+        encoded_messages = [
+            b'\x30\x10\xa1\x0e\xa0\x0c\x30\x0a\xa0\x03\x02\x01\x02\xa1\x03\x01'
+            b'\x01\xff',
+            b'\x30\x10\xa1\x0e\xa0\x0c\x30\x0a\xa0\x03\x02\x01\x02\xa1\x03\x01'
+            b'\x01\xff',
+            b'[{"b": {"c": [{"a": 2, "b": true}]}}]',
+            b'\x01\x01\x81\x01\x01\x80\x01\x02\xff',
+            b'\x30\x01\x80\x01\x02\x80',
+            b'\x30\x18\x08\x14',
+            b'<F><b><c><SEQUENCE><a>2</a><b><true /></b></SEQUENCE></c></b></'
+            b'F>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'F', decoded, encoded)
+
+        # H.
+        decoded = [True]
+
+        encoded_messages = [
+            b'\x30\x03\x01\x01\xff',
+            b'\x30\x03\x01\x01\xff',
+            b'[true]',
+            b'\x01\x01\xff',
+            b'\x30',
+            b'\x30',
+            b'<H><true /></H>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'H', decoded, encoded)
+
+        # I.
+        decoded = [True]
+
+        encoded_messages = [
+            b'\x30\x03\x01\x01\xff',
+            b'\x30\x03\x01\x01\xff',
+            b'[true]',
+            b'\x01\x01\xff',
+            b'\xc0',
+            b'\xc0',
+            b'<I><true /></I>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'I', decoded, encoded)
+
+        # S.
+        decoded = {'a': True, 'b': 7}
+
+        encoded_messages = [
+            b'\x30\x08\xa0\x03\x01\x01\xff\x81\x01\x07',
+            b'\x30\x08\xa0\x03\x01\x01\xff\x81\x01\x07',
+            b'{"a": true, "b": 7}',
+            b'\xff\x01\x07',
+            b'\x80\x01\x07',
+            b'\x80\x83\x80',
+            b'<S><a><true /></a><b>7</b></S>'
+        ]
+
+        for spec, codec, encoded in zip(specs, CODECS, encoded_messages):
+            self.encode_decode_codec(spec, codec, 'S', decoded, encoded)
+
 
 if __name__ == '__main__':
     unittest.main()
