@@ -1,3 +1,4 @@
+import sys
 import os
 import re
 import unittest
@@ -16,7 +17,7 @@ import asn1tools
 
 
 def remove_date_time(string):
-    return re.sub(r' \* This file was generated.*', '', string)
+    return re.sub(r'.* This file was generated.*', '', string)
 
 
 def read_file(filename):
@@ -679,12 +680,15 @@ exit
             'asn1tools',
             'generate_c_source',
             '--namespace', 'oer',
-            'tests/files/c_source.asn',
-            'examples/programming_types/programming_types.asn'
+            '--generate-fuzzer',
+            'tests/files/c_source/c_source.asn',
+            'tests/files/c_source/programming_types.asn'
         ]
 
         filename_h = 'oer.h'
         filename_c = 'oer.c'
+        fuzzer_filename_c = 'oer_fuzzer.c'
+        fuzzer_filename_mk = 'oer_fuzzer.mk'
 
         if os.path.exists(filename_h):
             os.remove(filename_h)
@@ -692,6 +696,12 @@ exit
         if os.path.exists(filename_c):
             os.remove(filename_c)
 
+        if os.path.exists(fuzzer_filename_c):
+            os.remove(fuzzer_filename_c)
+
+        if os.path.exists(fuzzer_filename_mk):
+            os.remove(fuzzer_filename_mk)
+
         with patch('sys.argv', argv):
             asn1tools._main()
 
@@ -701,16 +711,24 @@ exit
         self.assertEqual(
             read_file('tests/files/c_source/' + filename_c),
             read_file(filename_c))
+        self.assertEqual(
+            read_file('tests/files/c_source/' + fuzzer_filename_c),
+            read_file(fuzzer_filename_c))
+        self.assertEqual(
+            read_file('tests/files/c_source/' + fuzzer_filename_mk),
+            read_file(fuzzer_filename_mk))
 
     def test_command_line_generate_c_source_oer_minus(self):
         argv = [
             'asn1tools',
             'generate_c_source',
-            'tests/files/c_source-minus.asn'
+            'tests/files/c_source/c_source-minus.asn'
         ]
 
         filename_h = 'c_source-minus.h'
         filename_c = 'c_source-minus.c'
+        fuzzer_filename_c = 'c_source-minus_fuzzer.c'
+        fuzzer_filename_mk = 'c_source-minus_fuzzer.mk'
 
         if os.path.exists(filename_h):
             os.remove(filename_h)
@@ -718,6 +736,12 @@ exit
         if os.path.exists(filename_c):
             os.remove(filename_c)
 
+        if os.path.exists(fuzzer_filename_c):
+            os.remove(fuzzer_filename_c)
+
+        if os.path.exists(fuzzer_filename_mk):
+            os.remove(fuzzer_filename_mk)
+
         with patch('sys.argv', argv):
             asn1tools._main()
 
@@ -727,6 +751,8 @@ exit
         self.assertEqual(
             read_file('tests/files/c_source/' + filename_c),
             read_file(filename_c))
+        self.assertFalse(os.path.exists(fuzzer_filename_c))
+        self.assertFalse(os.path.exists(fuzzer_filename_mk))
 
     def test_command_line_generate_c_source_uper(self):
         argv = [
@@ -734,11 +760,14 @@ exit
             'generate_c_source',
             '--namespace', 'uper',
             '--codec', 'uper',
-            'tests/files/c_source.asn'
+            '--generate-fuzzer',
+            'tests/files/c_source/c_source.asn'
         ]
 
         filename_h = 'uper.h'
         filename_c = 'uper.c'
+        fuzzer_filename_c = 'uper_fuzzer.c'
+        fuzzer_filename_mk = 'uper_fuzzer.mk'
 
         if os.path.exists(filename_h):
             os.remove(filename_h)
@@ -746,6 +775,12 @@ exit
         if os.path.exists(filename_c):
             os.remove(filename_c)
 
+        if os.path.exists(fuzzer_filename_c):
+            os.remove(fuzzer_filename_c)
+
+        if os.path.exists(fuzzer_filename_mk):
+            os.remove(fuzzer_filename_mk)
+
         with patch('sys.argv', argv):
             asn1tools._main()
 
@@ -755,6 +790,65 @@ exit
         self.assertEqual(
             read_file('tests/files/c_source/' + filename_c),
             read_file(filename_c))
+        self.assertEqual(
+            read_file('tests/files/c_source/' + fuzzer_filename_c),
+            read_file(fuzzer_filename_c))
+        self.assertEqual(
+            read_file('tests/files/c_source/' + fuzzer_filename_mk),
+            read_file(fuzzer_filename_mk))
+
+    def test_command_line_generate_c_source(self):
+        specs = [
+            'boolean',
+            'octet_string'
+        ]
+
+        for spec in specs:
+            argv = [
+                'asn1tools',
+                'generate_c_source',
+                '--namespace', '{}_uper'.format(spec),
+                '--codec', 'uper',
+                'tests/files/c_source/{}.asn'.format(spec)
+            ]
+
+            filename_h = spec + '_uper.h'
+            filename_c = spec + '_uper.c'
+
+            if os.path.exists(filename_h):
+                os.remove(filename_h)
+
+            if os.path.exists(filename_c):
+                os.remove(filename_c)
+
+            with patch('sys.argv', argv):
+                asn1tools._main()
+
+            self.assertEqual(
+                read_file('tests/files/c_source/' + filename_h),
+                read_file(filename_h))
+            self.assertEqual(
+                read_file('tests/files/c_source/' + filename_c),
+                read_file(filename_c))
+
+    def test_command_line_generate_rust_source_uper(self):
+        argv = [
+            'asn1tools',
+            'generate_rust_source',
+            'tests/files/rust_source/rust_source.asn'
+        ]
+
+        filename_rs = 'rust_source.rs'
+
+        if os.path.exists(filename_rs):
+            os.remove(filename_rs)
+
+        with patch('sys.argv', argv):
+            asn1tools._main()
+
+        self.assertEqual(
+            read_file('tests/files/rust_source/' + filename_rs),
+            read_file(filename_rs))
 
 
 if __name__ == '__main__':
