@@ -43,10 +43,12 @@ class Asn1ToolsOerTest(Asn1ToolsBaseTest):
             "D ::= INTEGER (-2147483648..2147483647) "
             "E ::= INTEGER (-9223372036854775808..9223372036854775807) "
             "F ::= INTEGER (0..255) "
-            "G ::= INTEGER (0..65536) "
-            "H ::= INTEGER (0..4294967296) "
+            "G ::= INTEGER (0..65535) "
+            "H ::= INTEGER (0..4294967295) "
             "I ::= INTEGER (0..18446744073709551615) "
             "J ::= INTEGER (0..18446744073709551616) "
+            "K ::= INTEGER (1..MAX) "
+            "L ::= INTEGER (MIN..0) "
             "END",
             'oer')
 
@@ -60,6 +62,11 @@ class Asn1ToolsOerTest(Asn1ToolsBaseTest):
             ('C',             -2, b'\xff\xfe'),
             ('D',             -2, b'\xff\xff\xff\xfe'),
             ('E',             -2, b'\xff\xff\xff\xff\xff\xff\xff\xfe'),
+            ('F',            128, b'\x80'),
+            ('G',            128, b'\x00\x80'),
+            ('G',           1000, b'\x03\xe8'),
+            ('H',            128, b'\x00\x00\x00\x80'),
+            ('I',            128, b'\x00\x00\x00\x00\x00\x00\x00\x80'),
             ('B',              1, b'\x01'),
             ('C',              1, b'\x00\x01'),
             ('D',              1, b'\x00\x00\x00\x01'),
@@ -68,7 +75,12 @@ class Asn1ToolsOerTest(Asn1ToolsBaseTest):
             ('C',            127, b'\x00\x7f'),
             ('D',            127, b'\x00\x00\x00\x7f'),
             ('E',            127, b'\x00\x00\x00\x00\x00\x00\x00\x7f'),
-            ('J',              1, b'\x01\x01')
+            ('I',              1, b'\x00\x00\x00\x00\x00\x00\x00\x01'),
+            ('J',              1, b'\x01\x01'),
+            ('K',              1, b'\x01\x01'),
+            ('K',            127, b'\x01\x7f'),
+            ('K',            128, b'\x01\x80'),
+            ('L',           -128, b'\x01\x80')
         ]
 
         for type_name, decoded, encoded in datas:
@@ -253,6 +265,8 @@ class Asn1ToolsOerTest(Asn1ToolsBaseTest):
             "C ::= ENUMERATED { a(0), b(127) } "
             "D ::= ENUMERATED { a(0), ..., b(127) } "
             "E ::= ENUMERATED { a(-1), b(1234) } "
+            "F ::= ENUMERATED { a(-16777216), b(-8388608), c(-65536), "
+            "d(-32768), e(-128) } "
             "END",
             'oer')
 
@@ -264,7 +278,12 @@ class Asn1ToolsOerTest(Asn1ToolsBaseTest):
             ('D',                    'a', b'\x00'),
             ('D',                    'b', b'\x7f'),
             ('E',                    'a', b'\x81\xff'),
-            ('E',                    'b', b'\x82\x04\xd2')
+            ('E',                    'b', b'\x82\x04\xd2'),
+            ('F',                    'a', b'\x84\xff\x00\x00\x00'),
+            ('F',                    'b', b'\x83\x80\x00\x00'),
+            ('F',                    'c', b'\x83\xff\x00\x00'),
+            ('F',                    'd', b'\x82\x80\x00'),
+            ('F',                    'e', b'\x81\x80'),
         ]
 
         for type_name, decoded, encoded in datas:
@@ -303,7 +322,8 @@ class Asn1ToolsOerTest(Asn1ToolsBaseTest):
             "E ::= SEQUENCE { "
             "  a BOOLEAN, "
             "  ..., "
-            "  b BOOLEAN "
+            "  b BOOLEAN, "
+            "  c BOOLEAN "
             "} "
             "F ::= SEQUENCE { "
             "  a BOOLEAN, "
@@ -334,6 +354,11 @@ class Asn1ToolsOerTest(Asn1ToolsBaseTest):
             "  a E, "
             "  b INTEGER "
             "} "
+            "O ::= SEQUENCE { "
+            "  a BOOLEAN, "
+            "  ..., "
+            "  b NULL "
+            "} "
             "END",
             'oer')
 
@@ -345,8 +370,8 @@ class Asn1ToolsOerTest(Asn1ToolsBaseTest):
             ('D',            {'a': True}, b'\x00\xff'),
             ('E',            {'a': True}, b'\x00\xff'),
             ('E',
-             {'a': True, 'b': True},
-             b'\x80\xff\x02\x07\x80\x01\xff'),
+             {'a': True, 'b': True, 'c': True},
+             b'\x80\xff\x02\x06\xc0\x01\xff\x01\xff'),
             ('F',            {'a': True}, b'\x00\xff'),
             ('F',
              {'a': True, 'b': True},
@@ -363,7 +388,10 @@ class Asn1ToolsOerTest(Asn1ToolsBaseTest):
             ('L',                     {}, b'\x00'),
             ('N',
              {'a': {'a': True, 'b': True}, 'b': 5},
-             b'\x80\xff\x02\x07\x80\x01\xff\x01\x05')
+             b'\x80\xff\x02\x06\x80\x01\xff\x01\x05'),
+            ('O',
+             {'a': True, 'b': None},
+             b'\x80\xff\x02\x07\x80\x00')
         ]
 
         for type_name, decoded, encoded in datas:
@@ -421,6 +449,7 @@ class Asn1ToolsOerTest(Asn1ToolsBaseTest):
             "Foo DEFINITIONS AUTOMATIC TAGS ::= "
             "BEGIN "
             "A ::= SEQUENCE OF INTEGER "
+            "B ::= SEQUENCE (SIZE (129)) OF INTEGER "
             "END",
             'oer')
 
@@ -429,7 +458,8 @@ class Asn1ToolsOerTest(Asn1ToolsBaseTest):
             ('A',            [1, 2], b'\x01\x02\x01\x01\x01\x02'),
             ('A',
              1000 * [0],
-             b'\x02\x03\xe8' + 1000 * b'\x01\x00')
+             b'\x02\x03\xe8' + 1000 * b'\x01\x00'),
+            ('B',         129 * [2], b'\x01\x81' + 129 * b'\x01\x02')
         ]
 
         for type_name, decoded, encoded in datas:
@@ -491,6 +521,27 @@ class Asn1ToolsOerTest(Asn1ToolsBaseTest):
             str(cm.exception),
             "Expected choice member tag '80', but got '81'.")
 
+    def test_choice_default_tags(self):
+        foo = asn1tools.compile_string(
+            "Foo DEFINITIONS ::= "
+            "BEGIN "
+            "A ::= CHOICE { "
+            "  a CHOICE { "
+            "    aa INTEGER "
+            "  } "
+            "} "
+            "END",
+            'oer')
+
+        datas = [
+            ('A',          ('a', ('aa', 1)), b'\x02\x02\x01\x01')
+        ]
+
+        for type_name, decoded, encoded in datas:
+            # ToDo: This is a bug. Fix it.
+            with self.assertRaises(TypeError):
+                self.assert_encode_decode(foo, type_name, decoded, encoded)
+
     def test_uft8_string(self):
         foo = asn1tools.compile_string(
             "Foo DEFINITIONS AUTOMATIC TAGS ::= "
@@ -935,8 +986,7 @@ class Asn1ToolsOerTest(Asn1ToolsBaseTest):
 
     def test_c_source(self):
         files = [
-            'tests/files/c_source.asn',
-            'examples/programming_types/programming_types.asn'
+            'tests/files/c_source/c_source.asn'
         ]
         foo = asn1tools.compile_files(files, 'oer')
 
@@ -956,6 +1006,19 @@ class Asn1ToolsOerTest(Asn1ToolsBaseTest):
         with self.assertRaises(asn1tools.codecs.OutOfDataError):
             foo.decode('L', b'\xff\x00')
 
+    def test_not_support_decode_with_length(self):
+        foo = asn1tools.compile_string(
+            "Foo DEFINITIONS AUTOMATIC TAGS ::= "
+            "BEGIN "
+            "A ::= OCTET STRING "
+            "END",
+            'oer')
+
+        with self.assertRaises(NotImplementedError) as cm:
+            foo.decode_with_length('A', b'\x01\x23\x45\x67\x89\xab\xcd\xef')
+
+        self.assertEqual(str(cm.exception), "This codec does not support decode_with_length().")
+
 
 if __name__ == '__main__':
     unittest.main()
