@@ -58,8 +58,7 @@ class Asn1ToolsParseTest(unittest.TestCase):
         self.parse_and_verify('information_object')
 
     def test_parse_x683(self):
-        with self.assertRaises(AssertionError):
-            self.parse_and_verify('x683')
+        self.parse_and_verify('x683')
 
     def test_parse_x680(self):
         self.parse_and_verify('x680')
@@ -163,6 +162,9 @@ class Asn1ToolsParseTest(unittest.TestCase):
     def test_parse_import_imported(self):
         self.parse_and_verify('import_imported')
 
+    def test_parse_parameterization(self):
+        self.parse_and_verify('parameterization')
+
     def test_parse_imports_global_module_reference(self):
         actual = asn1tools.parse_string('A DEFINITIONS ::= BEGIN '
                                         'IMPORTS '
